@@ -9,13 +9,14 @@ fi
 
 function signal_descendants { pid=$1
     for child in `ps -o pid= --ppid $pid`; do
+        echo Sending $signal to $child
         kill -s $signal $child
         signal_descendants $child
     done
 }
 
 for rr_pid in `pidof rr` ; do
-    if grep -qz '^record$' /proc/$rr_pid/cmdline ; then
+    if cat /proc/$rr_pid/cmdline | tr '\0' '\n' | head -n2 | tail -n1 | grep -qz '\(^record$\)\|/'  ; then
         signal_descendants $rr_pid
     fi
 done
