@@ -4,10 +4,13 @@
 
 #include "AutoRemoteSyscalls.h"
 
+#include <limits.h>
+
 #include "rr/rr.h"
 
 #include "kernel_metadata.h"
 #include "log.h"
+#include "ReplaySession.h"
 #include "Session.h"
 #include "task.h"
 #include "util.h"
@@ -25,6 +28,9 @@ template <typename Arch> struct socketcall_args {
 } __attribute__((packed));
 
 void AutoRestoreMem::init(const uint8_t* mem, ssize_t num_bytes) {
+  ASSERT(remote.task(), !remote.regs().sp().is_null())
+      << "Memory parameters were disabled";
+
   len = num_bytes;
   saved_sp = remote.regs().sp();
 
@@ -49,14 +55,28 @@ AutoRestoreMem::~AutoRestoreMem() {
   remote.task()->set_regs(remote.regs());
 }
 
-AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t)
+AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
+                                       MemParamsEnabled enable_mem_params)
     : t(t),
       initial_regs(t->regs()),
       initial_ip(t->ip()),
       initial_sp(t->regs().sp()),
       pending_syscallno(-1) {
+  // We could use privilged_traced_syscall_ip() here, but we don't actually
+  // need privileges because tracee seccomp filters are modified to only
+  // produce PTRACE_SECCOMP_EVENTs that we ignore. And before the rr page is
+  // loaded, the privileged_traced_syscall_ip is not available.
   initial_regs.set_ip(t->vm()->traced_syscall_ip());
-  maybe_fix_stack_pointer();
+  if (enable_mem_params == ENABLE_MEMORY_PARAMS) {
+    maybe_fix_stack_pointer();
+  } else {
+    initial_regs.set_sp(remote_ptr<void>());
+  }
+}
+
+static bool is_usable_area(const KernelMapping& km) {
+  return (km.prot() & (PROT_READ | PROT_WRITE)) == (PROT_READ | PROT_WRITE) &&
+         (km.flags() & MAP_PRIVATE);
 }
 
 void AutoRemoteSyscalls::maybe_fix_stack_pointer() {
@@ -67,24 +87,23 @@ void AutoRemoteSyscalls::maybe_fix_stack_pointer() {
   remote_ptr<void> last_stack_byte = t->regs().sp() - 1;
   if (t->vm()->has_mapping(last_stack_byte)) {
     auto m = t->vm()->mapping_of(last_stack_byte);
-    if (m.second.is_stack() && m.first.start + 1024 <= t->regs().sp()) {
+    if (is_usable_area(m.map) && m.map.start() + 2048 <= t->regs().sp()) {
       // 'sp' is in a stack region and there's plenty of space there. No need
       // to fix anything.
       return;
     }
   }
 
-  Mapping found_stack;
-  auto find_stack = [&found_stack](const Mapping& m,
-                                   const MappableResource& r) {
-    if (r.is_stack()) {
-      found_stack = m;
+  MemoryRange found_stack;
+  for (auto m : t->vm()->maps()) {
+    if (is_usable_area(m.map)) {
+      found_stack = m.map;
+      break;
     }
   };
-  t->vm()->for_all_mappings(find_stack);
-  ASSERT(t, !found_stack.start.is_null()) << "No stack area found";
+  ASSERT(t, !found_stack.start().is_null()) << "No stack area found";
 
-  initial_regs.set_sp(found_stack.end);
+  initial_regs.set_sp(found_stack.end());
 }
 
 AutoRemoteSyscalls::~AutoRemoteSyscalls() { restore_state_to(t); }
@@ -96,20 +115,12 @@ void AutoRemoteSyscalls::restore_state_to(Task* t) {
   t->set_regs(initial_regs);
 }
 
-// TODO de-dup
-static void advance_syscall(Task* t) {
-  do {
-    t->cont_syscall();
-  } while (t->is_ptrace_seccomp_event() || SIGCHLD == t->pending_sig());
-  ASSERT(t, t->ptrace_event() == 0);
-}
-
-long AutoRemoteSyscalls::syscall_helper(SyscallWaiting wait, int syscallno,
+void AutoRemoteSyscalls::syscall_helper(SyscallWaiting wait, int syscallno,
                                         Registers& callregs) {
   callregs.set_syscallno(syscallno);
   t->set_regs(callregs);
 
-  advance_syscall(t);
+  t->advance_syscall();
 
   ASSERT(t, t->regs().ip() - callregs.ip() ==
                 syscall_instruction_length(t->arch()))
@@ -121,14 +132,13 @@ long AutoRemoteSyscalls::syscall_helper(SyscallWaiting wait, int syscallno,
 
   // Start running the syscall.
   pending_syscallno = syscallno;
-  t->cont_syscall_nonblocking();
+  t->resume_execution(RESUME_SYSCALL, RESUME_NONBLOCKING, RESUME_NO_TICKS);
   if (WAIT == wait) {
-    return wait_syscall(syscallno);
+    wait_syscall(syscallno);
   }
-  return 0;
 }
 
-long AutoRemoteSyscalls::wait_syscall(int syscallno) {
+void AutoRemoteSyscalls::wait_syscall(int syscallno) {
   ASSERT(t, pending_syscallno == syscallno || syscallno < 0);
 
   // Wait for syscall-exit trap.
@@ -138,8 +148,6 @@ long AutoRemoteSyscalls::wait_syscall(int syscallno) {
   ASSERT(t, t->regs().original_syscallno() == syscallno || syscallno < 0)
       << "Should be entering " << t->syscall_name(syscallno)
       << ", but instead at " << t->syscall_name(t->regs().original_syscallno());
-
-  return t->regs().syscall_result_signed();
 }
 
 SupportedArch AutoRemoteSyscalls::arch() const { return t->arch(); }
@@ -199,14 +207,13 @@ static int child_create_socket(AutoRemoteSyscalls& remote,
                                remote_ptr<socketcall_args<Arch> > sc_args) {
   int child_sock;
   if (sc_args.is_null()) {
-    child_sock = remote.syscall(Arch::socket, AF_UNIX, SOCK_STREAM, 0);
+    child_sock =
+        remote.infallible_syscall(Arch::socket, AF_UNIX, SOCK_STREAM, 0);
   } else {
     write_socketcall_args<Arch>(remote.task(), sc_args, AF_UNIX, SOCK_STREAM,
                                 0);
-    child_sock = remote.syscall(Arch::socketcall, SYS_SOCKET, sc_args);
-  }
-  if (child_sock < 0) {
-    FATAL() << "Failed to create child socket";
+    child_sock =
+        remote.infallible_syscall(Arch::socketcall, SYS_SOCKET, sc_args);
   }
   return child_sock;
 }
@@ -227,17 +234,9 @@ static void child_connect_socket(AutoRemoteSyscalls& remote,
   auto tmp_buf_end = buf_end;
   auto remote_dot = allocate(&tmp_buf_end, remote_buf, 2);
   remote.task()->write_mem(remote_dot.cast<char>(), ".", 2);
-  *cwd_fd = remote.syscall(syscall_number_for_open(Arch::arch()), remote_dot,
-                           O_PATH | O_DIRECTORY);
-  if (0 > *cwd_fd) {
-    FATAL() << "Failed to open() cwd in tracee; err=" << errno_name(-*cwd_fd);
-  }
-  int child_syscall_result =
-      remote.syscall(Arch::fchdir, RR_RESERVED_ROOT_DIR_FD);
-  if (0 > child_syscall_result) {
-    FATAL() << "Failed to fchdir() in tracee; err="
-            << errno_name(-child_syscall_result);
-  }
+  *cwd_fd = remote.infallible_syscall(syscall_number_for_open(Arch::arch()),
+                                      remote_dot, O_PATH | O_DIRECTORY);
+  remote.infallible_syscall(Arch::fchdir, RR_RESERVED_ROOT_DIR_FD);
 
   auto remote_addr = allocate<typename Arch::sockaddr_un>(&buf_end, remote_buf);
   remote.task()->write_mem(remote_addr, addr);
@@ -259,21 +258,6 @@ static void child_connect_socket(AutoRemoteSyscalls& remote,
                         callregs);
 }
 
-template <typename Arch>
-static void child_restore_cwd(AutoRemoteSyscalls& remote, int cwd_fd) {
-  // Restore child's cwd
-  int child_syscall_result = remote.syscall(Arch::fchdir, cwd_fd);
-  if (0 > child_syscall_result) {
-    FATAL() << "Failed to fchdir() to old cwd in tracee; err="
-            << errno_name(-child_syscall_result);
-  }
-  child_syscall_result = remote.syscall(Arch::close, cwd_fd);
-  if (0 > child_syscall_result) {
-    FATAL() << "Failed to close() cwd in tracee; err="
-            << errno_name(-child_syscall_result);
-  }
-}
-
 template <typename Arch>
 static void child_sendmsg(AutoRemoteSyscalls& remote,
                           AutoRestoreMem& remote_buf,
@@ -390,18 +374,21 @@ template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
     FATAL() << "Failed to create parent socket";
   }
   // Complete child's connect() syscall
-  int child_syscall_result = wait_syscall();
+  wait_syscall();
+  int child_syscall_result = t->regs().syscall_result_signed();
   if (child_syscall_result) {
     FATAL() << "Failed to connect() in tracee; err="
             << errno_name(-child_syscall_result);
   }
-  child_restore_cwd<Arch>(*this, cwd_fd);
+  infallible_syscall(Arch::fchdir, cwd_fd);
+  infallible_syscall(Arch::close, cwd_fd);
 
   // Listening socket not needed anymore
   close(listen_sock);
   unlink(path);
   child_sendmsg(*this, remote_buf, sc_args, sc_args_end, child_sock, fd);
-  child_syscall_result = wait_syscall();
+  wait_syscall();
+  child_syscall_result = t->regs().syscall_result_signed();
   if (0 >= child_syscall_result) {
     FATAL() << "Failed to sendmsg() in tracee; err="
             << errno_name(-child_syscall_result);
@@ -409,11 +396,7 @@ template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
   // Child may be waiting on our recvmsg().
   int our_fd = recvmsg_socket(sock);
 
-  child_syscall_result = syscall(Arch::close, child_sock);
-  if (0 > child_syscall_result) {
-    FATAL() << "Failed to close() in tracee; err="
-            << errno_name(-child_syscall_result);
-  }
+  child_syscall_result = infallible_syscall(Arch::close, child_sock);
   if (0 > close(sock)) {
     FATAL() << "Failed to close parent socket";
   }
@@ -425,16 +408,36 @@ ScopedFd AutoRemoteSyscalls::retrieve_fd(int fd) {
   RR_ARCH_FUNCTION(retrieve_fd_arch, arch(), fd);
 }
 
-remote_ptr<void> AutoRemoteSyscalls::mmap_syscall(remote_ptr<void> addr,
-                                                  size_t length, int prot,
-                                                  int flags, int child_fd,
-                                                  uint64_t offset_pages) {
-  if (has_mmap2_syscall(arch())) {
-    syscall(syscall_number_for_mmap2(arch()), addr, length, prot, flags,
-            child_fd, (off_t)offset_pages);
-  } else {
-    syscall(syscall_number_for_mmap(arch()), addr, length, prot, flags,
-            child_fd, offset_pages * page_size());
+remote_ptr<void> AutoRemoteSyscalls::infallible_mmap_syscall(
+    remote_ptr<void> addr, size_t length, int prot, int flags, int child_fd,
+    uint64_t offset_pages) {
+  // The first syscall argument is called "arg 1", so
+  // our syscall-arg-index template parameter starts
+  // with "1".
+  remote_ptr<void> ret =
+      has_mmap2_syscall(arch())
+          ? infallible_syscall_ptr(syscall_number_for_mmap2(arch()), addr,
+                                   length, prot, flags, child_fd,
+                                   (off_t)offset_pages)
+          : infallible_syscall_ptr(syscall_number_for_mmap(arch()), addr,
+                                   length, prot, flags, child_fd,
+                                   offset_pages * page_size());
+  if (flags & MAP_FIXED) {
+    ASSERT(t, addr == ret) << "MAP_FIXED at " << addr << " but got " << ret;
+  }
+  return ret;
+}
+
+void AutoRemoteSyscalls::check_syscall_result(int syscallno) {
+  long ret = t->regs().syscall_result_signed();
+  if (-4096 < ret && ret < 0) {
+    string extra_msg;
+    if (is_open_syscall(syscallno, arch())) {
+      extra_msg = " opening " + t->read_c_str(t->regs().arg1());
+    } else if (is_openat_syscall(syscallno, arch())) {
+      extra_msg = " opening " + t->read_c_str(t->regs().arg2());
+    }
+    ASSERT(t, false) << "Syscall " << syscall_name(syscallno, arch())
+                     << " failed with errno " << errno_name(-ret) << extra_msg;
   }
-  return t->regs().syscall_result();
 }
