@@ -16,10 +16,7 @@
 
 class TraceReader;
 
-enum OptionParameters {
-  NO_PARAMETER,
-  HAS_PARAMETER
-};
+enum OptionParameters { NO_PARAMETER, HAS_PARAMETER };
 struct OptionSpec {
   char short_name;
   const char* long_name;
@@ -54,7 +51,7 @@ public:
                            ParsedOption* out);
   template <size_t N>
   static bool parse_option(std::vector<std::string>& args,
-                           const OptionSpec (&option_specs)[N],
+                           const OptionSpec(&option_specs)[N],
                            ParsedOption* out) {
     return parse_option(args, option_specs, N, out);
   }
