@@ -103,7 +103,7 @@ static void process_syscall(Task* t, int syscallno) {
  * generated by a single-step) or a syscall is made.
  */
 DiversionSession::DiversionResult DiversionSession::diversion_step(
-    Task* t, RunCommand command) {
+    Task* t, RunCommand command, int signal_to_deliver) {
   assert(command != RUN_SINGLESTEP_FAST_FORWARD);
   assert_fully_initialized();
 
@@ -115,13 +115,20 @@ DiversionSession::DiversionResult DiversionSession::diversion_step(
     return result;
   }
 
+  if (t->syscallbuf_hdr) {
+    // Disable syscall buffering during diversions
+    t->syscallbuf_hdr->locked = 1;
+  }
+
   switch (command) {
     case RUN_CONTINUE:
       LOG(debug) << "Continuing to next syscall";
-      t->cont_sysemu();
+      t->resume_execution(RESUME_SYSEMU, RESUME_WAIT, RESUME_UNLIMITED_TICKS,
+                          signal_to_deliver);
       break;
     case RUN_SINGLESTEP:
-      t->cont_sysemu_singlestep();
+      t->resume_execution(RESUME_SYSEMU_SINGLESTEP, RESUME_WAIT,
+                          RESUME_UNLIMITED_TICKS, signal_to_deliver);
       LOG(debug) << "Stepping to next insn/syscall";
       break;
     default:
