@@ -26,11 +26,11 @@ static void replace_char(string& s, char c, char replacement) {
 }
 
 EmuFile::~EmuFile() {
-  LOG(debug) << "    EmuFs::~File(einode:" << est.st_ino << ")";
+  LOG(debug) << "    EmuFs::~File(einode:" << inode_ << ")";
 }
 
 EmuFile::shr_ptr EmuFile::clone() {
-  auto f = EmuFile::create(orig_path.c_str(), est);
+  auto f = EmuFile::create(orig_path.c_str(), device(), inode(), size_);
   // NB: this isn't the most efficient possible file copy, but
   // it's simple and not too slow.
   ifstream src(proc_path(), ifstream::binary);
@@ -45,35 +45,63 @@ string EmuFile::proc_path() const {
   return ss.str();
 }
 
-void EmuFile::update(const struct stat& st) {
-  assert(est.st_dev == st.st_dev && est.st_ino == st.st_ino);
-  if (est.st_size != st.st_size) {
-    resize_shmem_segment(file, st.st_size);
+void EmuFile::update(dev_t device, ino_t inode, uint64_t size) {
+  assert(device_ == device && inode_ == inode);
+  if (size_ != size) {
+    resize_shmem_segment(file, size);
   }
-  est = st;
+  size_ = size;
 }
 
 /*static*/ EmuFile::shr_ptr EmuFile::create(const string& orig_path,
-                                            const struct stat& est) {
+                                            dev_t orig_device, ino_t orig_inode,
+                                            uint64_t orig_file_size) {
   // Sanitize the mapped file path so that we can use it in a
   // leaf name.
   string path_tag(orig_path);
   replace_char(path_tag, '/', '\\');
 
   stringstream name;
-  name << "rr-emufs-" << getpid() << "-dev-" << est.st_dev << "-inode-"
-       << est.st_ino << "-" << path_tag;
-  shr_ptr f(new EmuFile(create_shmem_segment(name.str(), est.st_size), est,
-                        orig_path));
+  name << SHMEM_FS << "/rr-emufs-" << getpid() << "-dev-" << orig_device
+       << "-inode-" << orig_inode << "-" << path_tag;
+  string real_name = name.str().substr(0, 255);
+
+  ScopedFd fd =
+      open(real_name.c_str(), O_CREAT | O_EXCL | O_RDWR | O_CLOEXEC, 0600);
+  if (!fd.is_open()) {
+    FATAL() << "Failed to create shmem segment " << real_name;
+  }
+  /* Remove the fs name so that we don't have to worry about
+   * cleaning up this segment in error conditions. */
+  unlink(real_name.c_str());
+  resize_shmem_segment(fd, orig_file_size);
+
+  shr_ptr f(new EmuFile(std::move(fd), orig_path, real_name, orig_device,
+                        orig_inode, orig_file_size));
+
   LOG(debug) << "created emulated file for " << orig_path << " as "
              << name.str();
   return f;
 }
 
-EmuFile::EmuFile(ScopedFd&& fd, const struct stat& est, const string& orig_path)
-    : est(est), orig_path(orig_path), file(std::move(fd)), is_marked(false) {}
+EmuFile::EmuFile(ScopedFd&& fd, const string& orig_path,
+                 const string& real_path, dev_t orig_device, ino_t orig_inode,
+                 uint64_t orig_file_size)
+    : orig_path(orig_path),
+      tmp_path(real_path),
+      file(std::move(fd)),
+      size_(orig_file_size),
+      device_(orig_device),
+      inode_(orig_inode),
+      is_marked(false) {}
 
-EmuFile::shr_ptr EmuFs::at(const FileId& id) const { return files.at(id); }
+EmuFile::shr_ptr EmuFs::at(const KernelMapping& recorded_map) const {
+  return files.at(FileId(recorded_map));
+}
+
+bool EmuFs::has_file_for(const KernelMapping& recorded_map) const {
+  return files.find(FileId(recorded_map)) != files.end();
+}
 
 EmuFs::shr_ptr EmuFs::clone() {
   shr_ptr fs(new EmuFs());
@@ -135,35 +163,22 @@ void EmuFs::gc(const Session& session) {
     it->second->unmark();
   }
   for (auto it = garbage.begin(); it != garbage.end(); ++it) {
-    LOG(debug) << "  emufs gc reclaiming einode:" << it->disp_inode()
-               << "; fs name `" << files[*it]->emu_path() << "'";
+    LOG(debug) << "  emufs gc reclaiming einode:" << it->inode << "; fs name `"
+               << files[*it]->emu_path() << "'";
     files.erase(*it);
   }
 }
 
-EmuFile::shr_ptr EmuFs::get_or_create(const TraceMappedRegion& mf) {
-  FileId id(mf.stat(), mf.type() == TraceMappedRegion::MMAP
-                           ? PSEUDODEVICE_SHARED_MMAP_FILE
-                           : PSEUDODEVICE_SYSV_SHM);
+EmuFile::shr_ptr EmuFs::get_or_create(const KernelMapping& recorded_km,
+                                      uint64_t file_size) {
+  FileId id(recorded_km);
   auto it = files.find(id);
   if (it != files.end()) {
-    it->second->update(mf.stat());
+    it->second->update(recorded_km.device(), recorded_km.inode(), file_size);
     return it->second;
   }
-  auto vf = EmuFile::create(mf.file_name(), mf.stat());
-  files[id] = vf;
-  return vf;
-}
-
-EmuFile::shr_ptr EmuFs::create_anonymous(const FileId& id, size_t size) {
-  assert(files.find(id) == files.end());
-  struct stat fake_stat;
-  memset(&fake_stat, 0, sizeof(fake_stat));
-  fake_stat.st_ino = id.internal_inode();
-  fake_stat.st_size = size;
-  stringstream name;
-  name << "anonymous-" << id.internal_inode();
-  auto vf = EmuFile::create(name.str(), fake_stat);
+  auto vf = EmuFile::create(recorded_km.fsname(), recorded_km.device(),
+                            recorded_km.inode(), file_size);
   files[id] = vf;
   return vf;
 }
@@ -182,19 +197,20 @@ EmuFs::EmuFs() {}
 
 void EmuFs::mark_used_vfiles(Task* t, const AddressSpace& as,
                              size_t* nr_marked_files) {
-  for (auto& kv : as.memmap()) {
-    const MappableResource& r = kv.second;
-    LOG(debug) << "  examining " << r.fsname.c_str() << " ...";
+  for (auto m : as.maps()) {
+    LOG(debug) << "  examining " << m.map.fsname().c_str() << " ...";
 
-    auto id_ef = files.find(r.id);
+    FileId id(m.recorded_map);
+    auto id_ef = files.find(id);
     if (id_ef == files.end()) {
-      ASSERT(t, !r.is_shared_mmap_file());
+      // Mapping isn't relevant. Not all shared mappings get EmuFs entries
+      // (e.g. readonly shared mappings of certain system files, like fonts).
       continue;
     }
     auto ef = id_ef->second;
     if (!ef->marked()) {
       ef->mark();
-      LOG(debug) << "    marked einode:" << r.id.disp_inode();
+      LOG(debug) << "    marked einode:" << id.inode;
       ++*nr_marked_files;
       if (files.size() == *nr_marked_files) {
         LOG(debug) << "  (marked all files, bailing)";
@@ -203,21 +219,3 @@ void EmuFs::mark_used_vfiles(Task* t, const AddressSpace& as,
     }
   }
 }
-
-EmuFs::AutoGc::AutoGc(ReplaySession& session, SupportedArch arch, int syscallno,
-                      SyscallState state)
-    : session(session),
-      is_gc_point(session.emufs().size() > 0 && EXITING_SYSCALL == state &&
-                  (is_close_syscall(syscallno, arch) ||
-                   is_munmap_syscall(syscallno, arch))) {
-  if (is_gc_point) {
-    LOG(debug) << "emufs gc required because of syscall `"
-               << syscall_name(syscallno, arch) << "'";
-  }
-}
-
-EmuFs::AutoGc::~AutoGc() {
-  if (is_gc_point) {
-    session.gc_emufs();
-  }
-}
