@@ -8,6 +8,7 @@
 #include <string>
 #include <vector>
 
+#include "AddressSpace.h"
 #include "ScopedFd.h"
 #include "task.h"
 
@@ -60,28 +61,40 @@ public:
 
   ~EmuFile();
 
-  /**
-   * Return a copy of this file.  See |create()| for the meaning
-   * of |fs_tag|.
-   */
-  shr_ptr clone();
-
   /**
    * Return the fd of the real file backing this.
    */
   const ScopedFd& fd() const { return file; }
 
+  /**
+   * Return a pathname referring to the fd of this in this
+   * tracer's address space.  For example, "/proc/12345/fd/5".
+   */
+  std::string proc_path() const;
+
   /**
    * Return the path of the original file from recording, the
    * one this is emulating.
    */
   const std::string emu_path() const { return orig_path; }
 
+  const std::string real_path() const { return tmp_path; }
+
+  dev_t device() const { return device_; }
+  ino_t inode() const { return inode_; }
+
+private:
+  friend class EmuFs;
+
+  EmuFile(ScopedFd&& fd, const std::string& orig_path,
+          const std::string& real_path, dev_t device, ino_t inode,
+          uint64_t file_size);
+
   /**
-   * Return a pathname referring to the fd of this in this
-   * tracer's address space.  For example, "/proc/12345/fd/5".
+   * Return a copy of this file.  See |create()| for the meaning
+   * of |fs_tag|.
    */
-  std::string proc_path() const;
+  shr_ptr clone();
 
   /**
    * Mark/unmark/check to see if this file is marked.
@@ -92,11 +105,9 @@ public:
 
   /**
    * Ensure that the emulated file is sized to match a later
-   * stat() of it, |st|.
+   * stat() of it.
    */
-  void update(const struct stat& st);
-
-  const struct stat& stat() { return est; }
+  void update(dev_t device, ino_t inode, uint64_t size);
 
   /**
    * Create a new emulated file for |orig_path| that will
@@ -104,14 +115,15 @@ public:
    * uniquely identify this file among multiple EmuFs's that
    * might exist concurrently in this tracer process.
    */
-  static shr_ptr create(const std::string& orig_path, const struct stat& est);
+  static shr_ptr create(const std::string& orig_path, dev_t orig_device,
+                        ino_t orig_inode, uint64_t orig_file_size);
 
-private:
-  EmuFile(ScopedFd&& fd, const struct stat& est, const std::string& orig_path);
-
-  struct stat est;
   std::string orig_path;
+  std::string tmp_path;
   ScopedFd file;
+  uint64_t size_;
+  dev_t device_;
+  ino_t inode_;
   bool is_marked;
 
   EmuFile(const EmuFile&) = delete;
@@ -119,35 +131,30 @@ private:
 };
 
 class EmuFs {
-  typedef std::map<FileId, EmuFile::shr_ptr> FileMap;
-
 public:
   typedef std::shared_ptr<EmuFs> shr_ptr;
 
   /**
-   * Return the EmuFile defined by |id|, which must exist or
-   * this won't return.
+   * Return the EmuFile for |recorded_map|, which must exist or this won't
+   * return.
    */
-  EmuFile::shr_ptr at(const FileId& id) const;
+  EmuFile::shr_ptr at(const KernelMapping& recorded_map) const;
+
+  bool has_file_for(const KernelMapping& recorded_map) const;
 
   /**
-   * Return a copy of this fs such that |at()| and
-   * |get_or_create()| will return semantically identical
-   * results as this, and such that mutations of the returned fs
-   * won't affect this and vice versa.
+   * Return a copy of this fs such that |at()| and |get_or_create()| will
+   * return semantically identical results as this, and such that mutations of
+   * the returned fs won't affect this and vice versa.
    */
   shr_ptr clone();
 
   /**
-   * Return an emulated file representing the recorded file underlying |mf|.
+   * Return an emulated file representing the recorded shared mapping
+   * |recorded_km|.
    */
-  EmuFile::shr_ptr get_or_create(const TraceMappedRegion& mf);
-
-  /**
-   * Create an emulated file representing the shared anonymous mapping
-   * referenced by |id|.
-   */
-  EmuFile::shr_ptr create_anonymous(const FileId& id, size_t size);
+  EmuFile::shr_ptr get_or_create(const KernelMapping& recorded_km,
+                                 uint64_t file_size);
 
   /**
    * Dump information about this emufs to the "error" log.
@@ -159,20 +166,6 @@ public:
   /** Create and return a new emufs. */
   static shr_ptr create();
 
-  /**
-   * RAII helper that schedules an EmuFs GC when the exit of a given
-   * syscall may have dropped the last reference to an emulated file.
-   */
-  struct AutoGc {
-    AutoGc(ReplaySession& session, SupportedArch arch, int syscallno,
-           SyscallState state);
-    ~AutoGc();
-
-  private:
-    ReplaySession& session;
-    const bool is_gc_point;
-  };
-
   /**
    * Collect emulated files that aren't referenced by tracees.
    * Call this only when a tracee's (possibly shared) file table
@@ -192,6 +185,19 @@ private:
   void mark_used_vfiles(Task* t, const AddressSpace& as,
                         size_t* nr_marked_files);
 
+  struct FileId {
+    FileId(const KernelMapping& recorded_map)
+        : device(recorded_map.device()), inode(recorded_map.inode()) {}
+    bool operator<(const FileId& other) const {
+      return device < other.device ||
+             (device == other.device && inode < other.inode);
+    }
+    dev_t device;
+    ino_t inode;
+  };
+
+  typedef std::map<FileId, EmuFile::shr_ptr> FileMap;
+
   FileMap files;
 
   EmuFs(const EmuFs&) = delete;
