@@ -12,9 +12,24 @@
 #include "kernel_abi.h"
 #include "Registers.h"
 
+/**
+ * Events serve two purposes: tracking Task state during recording, and
+ * being stored in traces to guide replay. Some events are only used during
+ * recording and are never actually stored in traces (and are thus irrelevant
+ * to replay).
+ */
 enum EventType {
   EV_UNASSIGNED,
   EV_SENTINEL,
+  // TODO: this is actually a pseudo-pseudosignal: it will never
+  // appear in a trace, but is only used to communicate between
+  // different parts of the recorder code that should be
+  // refactored to not have to do that.
+  EV_NOOP,
+  EV_DESCHED,
+
+  // Events present in traces:
+
   // No associated data.
   EV_EXIT,
   // Tracee exited its sighandler.  We leave this breadcrumb so
@@ -25,20 +40,23 @@ enum EventType {
   // interrupted syscall wasn't restarted, so the interruption
   // record can be popped off the tracee's event stack.
   EV_INTERRUPTED_SYSCALL_NOT_RESTARTED,
-  // TODO: this is actually a pseudo-pseudosignal: it will never
-  // appear in a trace, but is only used to communicate between
-  // different parts of the recorder code that should be
-  // refactored to not have to do that.
-  EV_NOOP,
   // Scheduling signal interrupted the trace.
   EV_SCHED,
   EV_SEGV_RDTSC,
+  // Recorded syscallbuf data for one or more buffered syscalls.
   EV_SYSCALLBUF_FLUSH,
   EV_SYSCALLBUF_ABORT_COMMIT,
+  // The syscallbuf was reset to the empty state. We record this event
+  // later than it really happens, because during replay we must proceed to
+  // the event *after* a syscallbuf flush and then reset the syscallbuf,
+  // to ensure we don't reset it while preload code is still using the data.
   EV_SYSCALLBUF_RESET,
   // Syscall was entered, the syscall instruction was patched, and the
   // syscall was aborted. Resume execution at the patch.
   EV_PATCH_SYSCALL,
+  // Map memory pages due to a (future) memory access. This is associated
+  // with a mmap entry for the new pages.
+  EV_GROW_MAP,
   // The trace was terminated before all tasks exited, most
   // likely because the recorder was sent a terminating signal.
   // There are no more trace frames coming, so the best thing to
@@ -48,8 +66,6 @@ enum EventType {
   // "unstable" state in which we're not sure we can
   // synchronously wait for it to "really finish".
   EV_UNSTABLE_EXIT,
-  // Uses the .desched struct below.
-  EV_DESCHED,
   // Use .signal.
   EV_SIGNAL,
   EV_SIGNAL_DELIVERY,
@@ -57,13 +73,11 @@ enum EventType {
   // Use .syscall.
   EV_SYSCALL,
   EV_SYSCALL_INTERRUPTION,
+
   EV_LAST
 };
 
-enum HasExecInfo {
-  NO_EXEC_INFO,
-  HAS_EXEC_INFO
-};
+enum HasExecInfo { NO_EXEC_INFO, HAS_EXEC_INFO };
 
 /**
  * An encoding of the relevant bits of |struct event| that can be
@@ -123,44 +137,30 @@ struct BaseEvent {
  * unbounded amount of time).  After the syscall exits, rr advances
  * the tracee to where the desched is "disarmed" by the tracee.
  */
-enum DeschedState {
-  ARMING_DESCHED_EVENT,
-  IN_SYSCALL,
-  DISARMING_DESCHED_EVENT,
-  DISARMED_DESCHED_EVENT
-};
 struct DeschedEvent : public BaseEvent {
   /** Desched of |rec|. */
   DeschedEvent(const struct syscallbuf_record* rec, SupportedArch arch)
-      : BaseEvent(NO_EXEC_INFO, arch), rec(rec), state(IN_SYSCALL) {}
+      : BaseEvent(NO_EXEC_INFO, arch), rec(rec) {}
   // Record of the syscall that was interrupted by a desched
   // notification.  It's legal to reference this memory /while
   // the desched is being processed only/, because |t| is in the
   // middle of a desched, which means it's successfully
   // allocated (but not yet committed) this syscall record.
   const struct syscallbuf_record* rec;
-  DeschedState state;
 };
 
 /**
  * Signal events track signals through the delivery phase, and if the
  * signal finds a sighandler, on to the end of the handling face.
  */
-enum SignalDeterministic {
-  NONDETERMINISTIC_SIG = 0,
-  DETERMINISTIC_SIG = 1
-};
+enum SignalDeterministic { NONDETERMINISTIC_SIG = 0, DETERMINISTIC_SIG = 1 };
 struct SignalEvent : public BaseEvent {
   /**
    * Signal |signo| is the signum, and |deterministic| is true
    * for deterministically-delivered signals (see
    * record_signal.cc).
    */
-  SignalEvent(const siginfo_t& siginfo, SignalDeterministic deterministic,
-              SupportedArch arch)
-      : BaseEvent(HAS_EXEC_INFO, arch),
-        siginfo(siginfo),
-        deterministic(deterministic) {}
+  SignalEvent(const siginfo_t& siginfo, SupportedArch arch);
   SignalEvent(int signo, SignalDeterministic deterministic, SupportedArch arch)
       : BaseEvent(HAS_EXEC_INFO, arch), deterministic(deterministic) {
     memset(&siginfo, 0, sizeof(siginfo));
