@@ -2,21 +2,40 @@
 
 #include "FdTable.h"
 
+#include <limits.h>
+
 #include <unordered_set>
 
 #include "rr/rr.h"
 
+#include "log.h"
 #include "Session.h"
 #include "task.h"
 
 using namespace std;
 
+void FdTable::add_monitor(int fd, FileMonitor* monitor) {
+  // In the future we could support multiple monitors on an fd, but we don't
+  // need to yet.
+  assert(!is_monitoring(fd));
+  fds[fd] = FileMonitor::shr_ptr(monitor);
+  update_syscallbuf_fds_disabled(fd);
+}
+
+bool FdTable::allow_close(int fd) {
+  auto it = fds.find(fd);
+  if (it == fds.end()) {
+    return true;
+  }
+  return it->second->allow_close();
+}
+
 Switchable FdTable::will_write(Task* t, int fd) {
   auto it = fds.find(fd);
-  if (it != fds.end()) {
-    return it->second->will_write(t);
+  if (it == fds.end()) {
+    return ALLOW_SWITCH;
   }
-  return ALLOW_SWITCH;
+  return it->second->will_write(t);
 }
 
 void FdTable::did_write(Task* t, int fd,
@@ -52,6 +71,7 @@ static bool is_fd_monitored_in_any_task(AddressSpace* vm, int fd) {
 
 void FdTable::update_syscallbuf_fds_disabled(int fd) {
   assert(fd >= 0);
+  assert(task_set().size() > 0);
 
   unordered_set<AddressSpace*> vms_updated;
   // It's possible for tasks with different VMs to share this fd table.
@@ -65,22 +85,21 @@ void FdTable::update_syscallbuf_fds_disabled(int fd) {
 
     if (!t->syscallbuf_fds_disabled_child.is_null() &&
         fd < SYSCALLBUF_FDS_DISABLED_SIZE) {
-      bool disable =
-          is_fd_monitored_in_any_task(vm, fd) || RR_RESERVED_ROOT_DIR_FD == fd;
+      bool disable = is_fd_monitored_in_any_task(vm, fd);
       t->write_mem(t->syscallbuf_fds_disabled_child + fd, (char)disable);
     }
   }
 }
 
 void FdTable::init_syscallbuf_fds_disabled(Task* t) {
+  ASSERT(t, has_task(t));
+
   if (t->syscallbuf_fds_disabled_child.is_null()) {
     return;
   }
 
   char disabled[SYSCALLBUF_FDS_DISABLED_SIZE];
   memset(disabled, 0, sizeof(disabled));
-  assert(RR_RESERVED_ROOT_DIR_FD < SYSCALLBUF_FDS_DISABLED_SIZE);
-  disabled[RR_RESERVED_ROOT_DIR_FD] = 1;
 
   // It's possible that some tasks in this address space have a different
   // FdTable. We need to disable syscallbuf for an fd if any tasks for this
@@ -107,6 +126,8 @@ static bool is_fd_open(Task* t, int fd) {
 }
 
 void FdTable::update_for_cloexec(Task* t, TraceTaskEvent& event) {
+  ASSERT(t, has_task(t));
+
   vector<int> fds_to_close;
 
   if (t->session().is_recording()) {
