@@ -23,13 +23,15 @@ HelpCommand HelpCommand::help3("--help", nullptr);
 
 int HelpCommand::run(std::vector<std::string>& args) {
   if (args.size() == 0) {
-    return print_usage();
+    print_usage(stdout);
+    return 0;
   }
 
   Command* command = Command::command_for_name(args[0]);
   if (!command) {
-    return print_usage();
+    print_usage(stderr);
+    return 1;
   }
-  command->print_help(stderr);
+  command->print_help(stdout);
   return 0;
 }
