@@ -28,12 +28,11 @@ RecordCommand RecordCommand::singleton(
     "  -e, --num-events=<NUM>     maximum number of events (syscall \n"
     "                             enter/exit, signal, CPU interrupt, ...) \n"
     "                             to allow a task before descheduling it\n"
-    "  -i, --ignore-signal=<SIG>  block <SIG> from being delivered to "
-    "tracees.\n"
-    "                             Probably only useful for unit tests.\n"
-    "  -n, --no-syscall-buffer    disable the syscall buffer preload "
-    "library\n"
-    "                             even if it would otherwise be used\n"
+    "  -i, --ignore-signal=<SIG>  block <SIG> from being delivered to \n"
+    "                             tracees. Probably only useful for unit \n"
+    "                             tests.\n"
+    "  -n, --no-syscall-buffer    disable the syscall buffer preload \n"
+    "                             library even if it would otherwise be used\n"
     "  -u, --cpu-unbound          allow tracees to run on any virtual CPU.\n"
     "                             Default is to bind to CPU 0.  This option\n"
     "                             can cause replay divergence: use with\n"
@@ -133,12 +132,6 @@ static bool parse_record_arg(std::vector<std::string>& args,
 
 static bool term_request;
 
-static void terminate_recording(RecordSession& session, int status = 0) {
-  session.terminate_recording();
-  LOG(info) << "  exiting, goodbye.";
-  exit(status);
-}
-
 /**
  * A terminating signal was received.  Set the |term_request| bit to
  * terminate the trace at the next convenient point.
@@ -146,7 +139,7 @@ static void terminate_recording(RecordSession& session, int status = 0) {
  * If there's already a term request pending, then assume rr is wedged
  * and abort().
  */
-static void handle_termsig(int sig) {
+static void handle_SIGTERM(int sig) {
   if (term_request) {
     FATAL() << "Received termsig while an earlier one was pending.  We're "
                "probably wedged.";
@@ -156,21 +149,14 @@ static void handle_termsig(int sig) {
   term_request = true;
 }
 
-static void install_termsig_handlers(void) {
-  int termsigs[] = { SIGINT, SIGTERM };
-  for (size_t i = 0; i < array_length(termsigs); ++i) {
-    struct sigaction sa;
-    memset(&sa, 0, sizeof(sa));
-    sa.sa_handler = handle_termsig;
-    sigaction(termsigs[i], &sa, nullptr);
-  }
-}
+static void install_signal_handlers(void) {
+  struct sigaction sa;
+  memset(&sa, 0, sizeof(sa));
+  sa.sa_handler = handle_SIGTERM;
+  sigaction(SIGTERM, &sa, nullptr);
 
-/** If |term_request| is set, then terminate_recording(). */
-static void maybe_process_term_request(RecordSession& session) {
-  if (term_request) {
-    terminate_recording(session);
-  }
+  sa.sa_handler = SIG_IGN;
+  sigaction(SIGINT, &sa, nullptr);
 }
 
 static void setup_session_from_flags(RecordSession& session,
@@ -183,8 +169,6 @@ static void setup_session_from_flags(RecordSession& session,
 static int record(const vector<string>& args, const RecordFlags& flags) {
   LOG(info) << "Start recording...";
 
-  install_termsig_handlers();
-
   auto session = RecordSession::create(
       args,
       (flags.cpu_unbound ? RecordSession::CPU_UNBOUND : 0) |
@@ -192,41 +176,58 @@ static int record(const vector<string>& args, const RecordFlags& flags) {
       flags.extra_env);
   setup_session_from_flags(*session, flags);
 
+  // Install signal handlers after creating the session, to ensure they're not
+  // inherited by the tracee.
+  install_signal_handlers();
+
   RecordSession::RecordResult step_result;
-  while ((step_result = session->record_step()).status ==
-         RecordSession::STEP_CONTINUE) {
-    maybe_process_term_request(*session);
-  }
+  do {
+    step_result = session->record_step();
+  } while (step_result.status == RecordSession::STEP_CONTINUE && !term_request);
 
-  if (step_result.status == RecordSession::STEP_EXEC_FAILED) {
-    fprintf(stderr,
-            "\n"
-            "rr: error:\n"
-            "  Unexpected `write()' call from first tracee process.\n"
-            "  Most likely, the executable image `%s' is 64-bit, doesn't "
-            "exist, or\n"
-            "  isn't in your $PATH.  Terminating recording.\n"
-            "\n",
-            session->trace_writer().initial_exe().c_str());
-    terminate_recording(*session);
-  }
+  session->terminate_recording();
 
-  if (step_result.status == RecordSession::STEP_PERF_COUNTERS_UNAVAILABLE) {
-    fprintf(stderr, "\n"
-                    "rr: internal recorder error:\n"
-                    "  Performance counter doesn't seem to be working.  Are "
-                    "you perhaps\n"
-                    "  running rr in a VM but didn't enable perf-counter "
-                    "virtualization?\n");
-    terminate_recording(*session, EX_UNAVAILABLE);
-  }
+  switch (step_result.status) {
+    case RecordSession::STEP_CONTINUE:
+      // SIGINT or something like that interrupted us.
+      return 0x80 | SIGINT;
 
-  assert(step_result.status == RecordSession::STEP_EXITED);
-  LOG(info) << "Done recording -- cleaning up";
-  return step_result.exit_code;
+    case RecordSession::STEP_EXITED:
+      return step_result.exit_code;
+
+    case RecordSession::STEP_EXEC_FAILED:
+      fprintf(stderr,
+              "\n"
+              "rr: error:\n"
+              "  Unexpected `write()' call from first tracee process.\n"
+              "  Most likely, the executable image `%s' is 64-bit, doesn't "
+              "exist, or\n"
+              "  isn't in your $PATH.  Terminating recording.\n"
+              "\n",
+              session->trace_writer().initial_exe().c_str());
+      return EX_NOINPUT;
+
+    case RecordSession::STEP_PERF_COUNTERS_UNAVAILABLE:
+      fprintf(stderr, "\n"
+                      "rr: internal recorder error:\n"
+                      "  Performance counter doesn't seem to be working.  Are "
+                      "you perhaps\n"
+                      "  running rr in a VM but didn't enable perf-counter "
+                      "virtualization?\n");
+      return EX_UNAVAILABLE;
+
+    default:
+      assert(0 && "Unknown exit status");
+      return -1;
+  }
 }
 
 int RecordCommand::run(std::vector<std::string>& args) {
+  if (getenv("RUNNING_UNDER_RR")) {
+    fprintf(stderr, "rr: cannot run rr recording under rr. Exiting.\n");
+    return 1;
+  }
+
   RecordFlags flags;
   while (parse_record_arg(args, flags)) {
   }
