@@ -7,6 +7,7 @@
 #include <vector>
 
 #include "Scheduler.h"
+#include "SeccompFilterRewriter.h"
 #include "Session.h"
 #include "task.h"
 #include "TraceFrame.h"
@@ -19,14 +20,10 @@ public:
   /**
    * Create a recording session for the initial command line |argv|.
    */
-  enum {
-    DISABLE_SYSCALL_BUF = 0x01,
-    CPU_UNBOUND = 0x02
-  };
-  static shr_ptr create(const std::vector<std::string>& argv,
-                        uint32_t flags = 0,
-                        const std::vector<std::string>& extra_env =
-                            std::vector<std::string>());
+  enum { DISABLE_SYSCALL_BUF = 0x01, CPU_UNBOUND = 0x02 };
+  static shr_ptr create(
+      const std::vector<std::string>& argv, uint32_t flags = 0,
+      const std::vector<std::string>& extra_env = std::vector<std::string>());
 
   bool use_syscall_buffer() const { return use_syscall_buffer_; }
   void set_ignore_sig(int ignore_sig) { this->ignore_sig = ignore_sig; }
@@ -73,18 +70,12 @@ public:
 
   Scheduler& scheduler() { return scheduler_; }
 
-private:
-  RecordSession(const std::vector<std::string>& argv,
-                const std::vector<std::string>& envp, const std::string& cwd,
-                uint32_t flags);
+  SeccompFilterRewriter& seccomp_filter_rewriter() {
+    return seccomp_filter_rewriter_;
+  }
 
-  virtual void on_create(Task* t);
+  enum ContinueType { DONT_CONTINUE = 0, CONTINUE, CONTINUE_SYSCALL };
 
-  enum ContinueType {
-    DONT_CONTINUE = 0,
-    CONTINUE,
-    CONTINUE_SYSCALL
-  };
   struct StepState {
     // Continue with this continuation type.
     ContinueType continue_type;
@@ -94,6 +85,13 @@ private:
         : continue_type(continue_type), continue_sig(0) {}
   };
 
+private:
+  RecordSession(const std::vector<std::string>& argv,
+                const std::vector<std::string>& envp, const std::string& cwd,
+                uint32_t flags);
+
+  virtual void on_create(Task* t);
+
   void check_perf_counters_working(Task* t, RecordResult* step_result);
   bool handle_ptrace_event(Task* t, StepState* step_state);
   bool handle_signal_event(Task* t, StepState* step_state);
@@ -110,6 +108,7 @@ private:
   Scheduler scheduler_;
   Task* last_recorded_task;
   TaskGroup::shr_ptr initial_task_group;
+  SeccompFilterRewriter seccomp_filter_rewriter_;
 
   int ignore_sig;
   Switchable last_task_switchable;
