@@ -62,10 +62,8 @@ void Session::post_exec() {
     return;
   }
   tracees_consistent = true;
-  // Reset ticks for all Tasks (there should only be one).
-  for (auto task = tasks().begin(); task != tasks().end(); ++task) {
-    task->second->flush_inconsistent_state();
-  }
+  assert(tasks().size() == 1);
+  tasks().begin()->second->flush_inconsistent_state();
 }
 
 AddressSpace::shr_ptr Session::create_vm(Task* t, const std::string& exe,
@@ -84,12 +82,12 @@ AddressSpace::shr_ptr Session::clone(Task* t, AddressSpace::shr_ptr vm) {
   AddressSpace::shr_ptr as;
   if (this == vm->session()) {
     as = AddressSpace::shr_ptr(
-        new AddressSpace(t, *vm, t->rec_tid, t->tuid().serial(), 0));
+        new AddressSpace(this, *vm, t->rec_tid, t->tuid().serial(), 0));
   } else {
-    as = AddressSpace::shr_ptr(new AddressSpace(
-        t, *vm, vm->uid().tid(), vm->uid().serial(), vm->uid().exec_count()));
+    as = AddressSpace::shr_ptr(new AddressSpace(this, *vm, vm->uid().tid(),
+                                                vm->uid().serial(),
+                                                vm->uid().exec_count()));
   }
-  as->session_ = this;
   vm_map[as->uid()] = as.get();
   return as;
 }
@@ -134,7 +132,7 @@ Task* Session::clone(Task* p, int flags, remote_ptr<void> stack,
 }
 
 Task* Session::find_task(pid_t rec_tid) const {
-  assert_fully_initialized();
+  finish_initializing();
   auto it = tasks().find(rec_tid);
   return tasks().end() != it ? it->second : nullptr;
 }
@@ -145,7 +143,7 @@ Task* Session::find_task(const TaskUid& tuid) const {
 }
 
 TaskGroup* Session::find_task_group(const TaskGroupUid& tguid) const {
-  assert_fully_initialized();
+  finish_initializing();
   auto it = task_group_map.find(tguid);
   if (task_group_map.end() == it) {
     return nullptr;
@@ -154,7 +152,7 @@ TaskGroup* Session::find_task_group(const TaskGroupUid& tguid) const {
 }
 
 AddressSpace* Session::find_address_space(const AddressSpaceUid& vmuid) const {
-  assert_fully_initialized();
+  finish_initializing();
   auto it = vm_map.find(vmuid);
   if (vm_map.end() == it) {
     return nullptr;
@@ -165,25 +163,46 @@ AddressSpace* Session::find_address_space(const AddressSpaceUid& vmuid) const {
 void Session::kill_all_tasks() {
   for (auto& v : task_map) {
     Task* t = v.second;
+
+    if (!t->is_stopped) {
+      // During recording we might be aborting the recording, in which case
+      // one or more tasks might not be stopped. We haven't got any really
+      // good options here so we'll just skip detaching and try killing
+      // it with SIGKILL below. rr will usually exit immediatley after this
+      // so the likelihood that we'll leak a zombie task isn't too bad.
+      continue;
+    }
+
     if (!t->stable_exit) {
       /*
        * Prepare to forcibly kill this task by detaching it first. To ensure
        * the task doesn't continue executing, we first set its ip() to an
-       * invalid
-       * value. We need to do this for all tasks in the Session before kill()
-       * is guaranteed to work properly. SIGKILL on ptrace-attached tasks seems
-       * to not work very well, and after sending SIGKILL we can't seem to
+       * invalid value. We need to do this for all tasks in the Session before
+       * kill() is guaranteed to work properly. SIGKILL on ptrace-attached tasks
+       * seems to not work very well, and after sending SIGKILL we can't seem to
        * reliably detach.
        */
       LOG(debug) << "safely detaching from " << t->tid << " ...";
       // Detaching from the process lets it continue. We don't want a replaying
       // process to perform syscalls or do anything else observable before we
-      // get around to SIGKILLing it. So we move its ip() to an invalid
-      // address. If it does continue, it will probably get a fatal signal.
-      // We don't install real signal handlers in replayed processes so there's
-      // no way it could handle the signal and continue.
+      // get around to SIGKILLing it. So we move its ip() to an address
+      // which will cause it to do an exit() syscall if it runs at all.
+      // We used to set this to an invalid address, but that causes a SIGSEGV
+      // to be raised which can cause core dumps after we detach from ptrace.
+      // Making the process undumpable with PR_SET_DUMPABLE turned out not to
+      // be practical because that has a side effect of triggering various
+      // security measures blocking inspection of the process (PTRACE_ATTACH,
+      // access to /proc/<pid>/fd).
+      // Disabling dumps via setrlimit(RLIMIT_CORE, 0) doesn't stop dumps
+      // if /proc/sys/kernel/core_pattern is set to pipe the core to a process
+      // (e.g. to systemd-coredump).
+      // We also tried setting ip() to an address that does an infinite loop,
+      // but that leaves a runaway process if something happens to kill rr
+      // after detaching but before we get a chance to SIGKILL the tracee.
       Registers r = t->regs();
-      r.set_ip(intptr_t(-1));
+      r.set_ip(t->vm()->privileged_traced_syscall_ip());
+      r.set_syscallno(syscall_number_for_exit(r.arch()));
+      r.set_arg1(0);
       t->set_regs(r);
       long result;
       do {
@@ -208,7 +227,13 @@ void Session::kill_all_tasks() {
       LOG(debug) << "sending SIGKILL to " << t->tid << " ...";
       // If we haven't already done a stable exit via syscall,
       // kill the task and note that the entire task group is unstable.
-      t->tgkill(SIGKILL);
+      // The task may already have exited due to the preparation above,
+      // so we might accidentally shoot down the wrong task :-(, but we
+      // have to do this because the task might be in a state where it's not
+      // going to run and exit by itself.
+      // Linux doesn't seem to give us a reliable way to detach and kill
+      // the tracee without races.
+      syscall(SYS_tgkill, t->real_tgid(), t->tid, SIGKILL);
       t->task_group()->destabilize();
     }
 
@@ -307,37 +332,36 @@ void Session::assert_fully_initialized() const {
   assert(!clone_completion && "Session not fully initialized");
 }
 
-void Session::finish_initializing() {
+void Session::finish_initializing() const {
   if (!clone_completion) {
     return;
   }
 
+  Session* self = const_cast<Session*>(this);
   for (auto& tgleader : clone_completion->task_groups) {
     AutoRemoteSyscalls remote(tgleader.clone_leader);
     for (auto& tgmember : tgleader.member_states) {
       Task* t_clone =
           Task::os_clone_into(tgmember, tgleader.clone_leader, remote);
-      on_create(t_clone);
+      self->on_create(t_clone);
       t_clone->copy_state(tgmember);
     }
     tgleader.clone_leader->copy_state(tgleader.clone_leader_state);
   }
 
-  clone_completion = nullptr;
+  self->clone_completion = nullptr;
 }
 
 static void remap_shared_mmap(AutoRemoteSyscalls& remote, EmuFs& dest_emu_fs,
-                              const Mapping& m, const MappableResource& r) {
-  LOG(debug) << "    remapping shared region at " << m.start << "-" << m.end;
-  remote.syscall(syscall_number_for_munmap(remote.arch()), m.start,
-                 m.num_bytes());
-  // NB: we don't have to unmap then re-map |t->vm()|'s idea of
-  // the emulated file mapping.  Though we'll be remapping the
-  // *real* OS mapping in |t| to a different file, that new
-  // mapping still refers to the same *emulated* file, with the
-  // same emulated metadata.
+                              const AddressSpace::Mapping& m_in_mem) {
+  AddressSpace::Mapping m = m_in_mem;
 
-  auto emufile = dest_emu_fs.at(r.id);
+  LOG(debug) << "    remapping shared region at " << m.map.start() << "-"
+             << m.map.end();
+  remote.infallible_syscall(syscall_number_for_munmap(remote.arch()),
+                            m.map.start(), m.map.size());
+
+  auto emufile = dest_emu_fs.at(m.recorded_map);
   // TODO: this duplicates some code in replay_syscall.cc, but
   // it's somewhat nontrivial to factor that code out.
   int remote_fd;
@@ -347,25 +371,33 @@ static void remap_shared_mmap(AutoRemoteSyscalls& remote, EmuFs& dest_emu_fs,
     // Always open the emufs file O_RDWR, even if the current mapping prot
     // is read-only. We might mprotect it to read-write later.
     // skip leading '/' since we want the path to be relative to the root fd
-    remote_fd =
-        remote.syscall(syscall_number_for_openat(remote.arch()),
-                       RR_RESERVED_ROOT_DIR_FD, child_path.get() + 1, O_RDWR);
+    remote_fd = remote.infallible_syscall(
+        syscall_number_for_openat(remote.arch()), RR_RESERVED_ROOT_DIR_FD,
+        child_path.get() + 1, O_RDWR);
     if (0 > remote_fd) {
       FATAL() << "Couldn't open " << path << " in tracee";
     }
   }
+  struct stat real_file = remote.task()->stat_fd(remote_fd);
+  string real_file_name = remote.task()->file_name_of_fd(remote_fd);
   // XXX this condition is x86/x64-specific, I imagine.
-  remote_ptr<void> addr =
-      remote.mmap_syscall(m.start, m.num_bytes(), m.prot,
-                          // The remapped segment *must* be
-                          // remapped at the same address,
-                          // or else many things will go
-                          // haywire.
-                          (m.flags & ~MAP_ANONYMOUS) | MAP_FIXED, remote_fd,
-                          m.offset / page_size());
-  ASSERT(remote.task(), addr == m.start);
+  remote.infallible_mmap_syscall(m.map.start(), m.map.size(), m.map.prot(),
+                                 // The remapped segment *must* be
+                                 // remapped at the same address,
+                                 // or else many things will go
+                                 // haywire.
+                                 (m.map.flags() & ~MAP_ANONYMOUS) | MAP_FIXED,
+                                 remote_fd,
+                                 m.map.file_offset_bytes() / page_size());
 
-  remote.syscall(syscall_number_for_close(remote.arch()), remote_fd);
+  // We update the AddressSpace mapping too, since that tracks the real file
+  // name and we need to update that.
+  remote.task()->vm()->map(m.map.start(), m.map.size(), m.map.prot(),
+                           m.map.flags(), m.map.file_offset_bytes(),
+                           real_file_name, real_file.st_dev, real_file.st_ino,
+                           &m.recorded_map);
+
+  remote.infallible_syscall(syscall_number_for_close(remote.arch()), remote_fd);
 }
 
 void Session::copy_state_to(Session& dest, EmuFs& dest_emu_fs) {
@@ -390,13 +422,11 @@ void Session::copy_state_to(Session& dest, EmuFs& dest_emu_fs) {
 
     {
       AutoRemoteSyscalls remote(group.clone_leader);
-      for (auto& kv : group.clone_leader->vm()->memmap()) {
-        const Mapping& m = kv.first;
-        const MappableResource& r = kv.second;
-        if (!r.is_shared_mmap_file()) {
-          continue;
+      for (auto m : group.clone_leader->vm()->maps()) {
+        if ((m.recorded_map.flags() & MAP_SHARED) &&
+            dest_emu_fs.has_file_for(m.recorded_map)) {
+          remap_shared_mmap(remote, dest_emu_fs, m);
         }
-        remap_shared_mmap(remote, dest_emu_fs, m, r);
       }
 
       for (auto t : group_leader->task_group()->task_set()) {
@@ -410,10 +440,6 @@ void Session::copy_state_to(Session& dest, EmuFs& dest_emu_fs) {
     }
 
     group.clone_leader_state = group_leader->capture_state();
-    // Close perfcounters for now. They will be automatically reopened
-    // when we next run this task (if ever). This reduces the numer of
-    // file descriptors we need to have open.
-    group.clone_leader->hpc.stop();
   }
   dest.clone_completion = move(completion);
 
