@@ -33,7 +33,8 @@ struct BreakStatus {
         signal(0),
         breakpoint_hit(false),
         singlestep_complete(false),
-        approaching_ticks_target(false) {}
+        approaching_ticks_target(false),
+        task_exit(false) {}
 
   // The triggering Task. This may be different from session->current_task()
   // when replay switches to a new task when ReplaySession::replay_step() ends.
@@ -51,6 +52,8 @@ struct BreakStatus {
   // True when we stopped because we got too close to the specified ticks
   // target.
   bool approaching_ticks_target;
+  // True when we stopped because |task| is about to exit.
+  bool task_exit;
 
   bool any_break() {
     return !watchpoints_hit.empty() || signal || breakpoint_hit ||
@@ -170,7 +173,7 @@ public:
 
   /** Return the set of Tasks being tracekd in this session. */
   const TaskMap& tasks() const {
-    assert_fully_initialized();
+    finish_initializing();
     return task_map;
   }
 
@@ -222,8 +225,8 @@ protected:
 
   struct CloneCompletion;
   // Call this before doing anything that requires access to the full set
-  // of tasks (i.e., almost anything!).
-  void finish_initializing();
+  // of tasks (i.e., almost anything!). Not really const!
+  void finish_initializing() const;
   void assert_fully_initialized() const;
 
   AddressSpaceMap vm_map;
