@@ -12,8 +12,10 @@
 #include <asm/ldt.h>
 #include <elf.h>
 #include <fcntl.h>
+#include <linux/capability.h>
 #include <linux/ethtool.h>
 #include <linux/filter.h>
+#include <linux/futex.h>
 #include <linux/ipc.h>
 #include <linux/msg.h>
 #include <linux/net.h>
@@ -27,6 +29,7 @@
 #include <signal.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <sound/asound.h>
 #include <sys/epoll.h>
 #include <sys/ioctl.h>
 #include <sys/quota.h>
@@ -46,13 +49,10 @@
 
 #include <vector>
 
+class remote_code_ptr;
 class Task;
 
-enum SupportedArch {
-  x86,
-  x86_64,
-  SupportedArch_MAX = x86_64
-};
+enum SupportedArch { x86, x86_64, SupportedArch_MAX = x86_64 };
 
 namespace rr {
 
@@ -101,6 +101,7 @@ struct KernelConstants {
   typedef uint32_t socklen_t;
   typedef uint64_t dev_t;
   typedef uint32_t mode_t;
+  typedef int32_t __kernel_timer_t;
 };
 
 // These duplicate the matching F_* constants for commands for fcntl, with two
@@ -132,6 +133,7 @@ struct FcntlConstants {
     GETOWN_EX = 16,
     // Linux-specific operations
     DUPFD_CLOEXEC = 0x400 + 6,
+    ADD_SEALS = 0x400 + 9
   };
 };
 
@@ -249,6 +251,8 @@ struct BaseArch : public wordsize, public FcntlConstants {
   typedef signed_int __kernel_pid_t;
   typedef int64_t __kernel_loff_t;
 
+  typedef unsigned_int __u32;
+
   template <typename T> struct ptr {
     typedef T Referent;
     unsigned_word val;
@@ -601,6 +605,19 @@ struct BaseArch : public wordsize, public FcntlConstants {
   };
   RR_VERIFY_TYPE(user_desc);
 
+  struct __user_cap_header_struct {
+    __u32 version;
+    int pid;
+  };
+  RR_VERIFY_TYPE(__user_cap_header_struct);
+
+  struct __user_cap_data_struct {
+    __u32 effective;
+    __u32 permitted;
+    __u32 inheritable;
+  };
+  RR_VERIFY_TYPE(__user_cap_data_struct);
+
   // This structure uses fixed-size fields, but the padding rules
   // for 32-bit vs. 64-bit architectures dictate that it be
   // defined in full.
@@ -960,9 +977,7 @@ struct BaseArch : public wordsize, public FcntlConstants {
   // The 'size' parameter to pass to rt_sigaction. Only this value works,
   // even though sizeof(sigset_t) > 8 (it's actually 128 with kernel 3.16,
   // as above).
-  enum {
-    sigaction_sigset_size = 8
-  };
+  enum { sigaction_sigset_size = 8 };
 
   struct tms {
     clock_t tms_utime;
@@ -1081,11 +1096,6 @@ struct BaseArch : public wordsize, public FcntlConstants {
   };
   RR_VERIFY_TYPE(sched_param);
 
-  typedef struct {
-    unsigned_long __bits[1024 / (8 * sizeof(unsigned_long))];
-  } cpu_set_t;
-  RR_VERIFY_TYPE(cpu_set_t);
-
   static void* cmsg_data(cmsghdr* cmsg) { return cmsg + 1; }
   static size_t cmsg_align(size_t len) {
     return (len + sizeof(size_t) - 1) & ~(sizeof(size_t) - 1);
@@ -1145,6 +1155,31 @@ struct BaseArch : public wordsize, public FcntlConstants {
     ptr<sock_filter> filter;
   };
   RR_VERIFY_TYPE(sock_fprog);
+
+  struct robust_list {
+    ptr<robust_list> next;
+  };
+  RR_VERIFY_TYPE(robust_list);
+
+  struct robust_list_head {
+    robust_list list;
+    signed_long futex_offset;
+    ptr<robust_list> list_op_pending;
+  };
+  RR_VERIFY_TYPE(robust_list_head);
+
+  struct snd_ctl_card_info {
+    int card;
+    int pad;
+    unsigned char id[16];
+    unsigned char driver[16];
+    unsigned char name[32];
+    unsigned char longname[80];
+    unsigned char reserved_[16];
+    unsigned char mixername[80];
+    unsigned char components[128];
+  };
+  RR_VERIFY_TYPE(snd_ctl_card_info);
 };
 
 struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
@@ -1323,6 +1358,8 @@ struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
     uint64_t rdx;
     uint64_t rsi;
     uint64_t rdi;
+    // Unsigned type matches <sys/user.h>, but we need to treat this as
+    // signed in practice.
     uint64_t orig_rax;
     uint64_t rip;
     uint32_t cs;
@@ -1443,7 +1480,7 @@ struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
 /**
  * Return true if |ptr| in task |t| points to an invoke-syscall instruction.
  */
-bool is_at_syscall_instruction(Task* t, remote_ptr<uint8_t> ptr);
+bool is_at_syscall_instruction(Task* t, remote_code_ptr ptr);
 
 /**
  * Return the code bytes of an invoke-syscall instruction. The vector must
