@@ -84,9 +84,9 @@ const char* ptrace_req_name(int request) {
 
 const char* signal_name(int sig) {
   /* strsignal() would be nice to use here, but it provides TMI. */
-  if (SIGRTMIN <= sig && sig <= SIGRTMAX) {
+  if (32 <= sig && sig <= 64) {
     static __thread char buf[] = "SIGRT00000000";
-    snprintf(buf, sizeof(buf) - 1, "SIGRT%d", sig - SIGRTMIN);
+    snprintf(buf, sizeof(buf) - 1, "SIGRT%d", sig);
     return buf;
   }
 
@@ -127,18 +127,12 @@ const char* signal_name(int sig) {
   }
 }
 
-#include "IsAlwaysEmulatedSyscall.generated"
-
-bool is_always_emulated_syscall(int syscall, SupportedArch arch) {
-  RR_ARCH_FUNCTION(is_always_emulated_syscall_arch, arch, syscall);
-}
-
 bool is_sigreturn(int syscallno, SupportedArch arch) {
   return is_sigreturn_syscall(syscallno, arch) ||
          is_rt_sigreturn_syscall(syscallno, arch);
 }
 
-const char* errno_name(int err) {
+string errno_name(int err) {
   switch (err) {
     case 0:
       return "SUCCESS";
@@ -273,8 +267,11 @@ const char* errno_name(int err) {
       CASE(ENOTRECOVERABLE);
       CASE(ERFKILL);
       CASE(EHWPOISON);
-    default:
-      return "???errno";
+    default: {
+      char buf[100];
+      sprintf(buf, "errno(%d)", err);
+      return string(buf);
+    }
   }
 }
 
