@@ -34,13 +34,6 @@ const char* ptrace_req_name(int request);
  */
 const char* signal_name(int sig);
 
-/**
- * Return true iff replaying |syscall| will never ever require
- * actually executing it, i.e. replay of |syscall| is always
- * emulated.
- */
-bool is_always_emulated_syscall(int syscall, SupportedArch arch);
-
 /**
  * Return true if this is some kind of sigreturn syscall.
  */
@@ -49,7 +42,7 @@ bool is_sigreturn(int syscall, SupportedArch arch);
 /**
  * Return the symbolic error name (e.g. "EINVAL") for errno.
  */
-const char* errno_name(int err);
+std::string errno_name(int err);
 
 /**
  * Return the symbolic name (e.g. "SI_USER") for an si_code.
