@@ -8,13 +8,7 @@
 #include "Flags.h"
 #include "task.h"
 
-enum LogLevel {
-  LOG_fatal,
-  LOG_error,
-  LOG_warn,
-  LOG_info,
-  LOG_debug
-};
+enum LogLevel { LOG_fatal, LOG_error, LOG_warn, LOG_info, LOG_debug };
 
 inline static bool logging_enabled_for(LogLevel level) {
   switch (level) {
