@@ -2,6 +2,7 @@
 
 #include "main.h"
 
+#include <limits.h>
 #include <linux/version.h>
 #include <stdio.h>
 #include <string.h>
@@ -81,7 +82,7 @@ void check_performance_settings() {
             "governor\n"
             "    by running the following commands:\n"
             "\n"
-            "    $ sudo yum install kernel-tools\n"
+            "    $ sudo dnf install kernel-tools\n"
             "    $ sudo cpupower frequency-set -g performance\n"
             "\n",
             governor);
@@ -131,10 +132,10 @@ void print_usage(FILE* out) {
       "/proc/maps\n"
       "  -E, --fatal-errors         any warning or error that is printed is\n"
       "                             treated as fatal\n"
-      "  -M, --mark-stdio           mark stdio writes with [rr.<EVENT-NO>],\n"
-      "                             where EVENT-NO is the global trace time "
-      "at\n"
-      "                             which the write occures.\n"
+      "  -M, --mark-stdio           mark stdio writes with [rr <PID> <EV>]\n"
+      "                             where EV is the global trace time at\n"
+      "                             which the write occurs and PID is the pid\n"
+      "                             of the process it occurs in.\n"
       "  -N, --version              print the version number and exit\n"
       "  -S, --suppress-environment-warnings\n"
       "                             suppress warnings about issues in the\n"
