@@ -5,13 +5,6 @@
 
 #include "task.h"
 
-/**
- * Call this just before the recorder is going to store a
- * syscall-entry event.  If any data needs to be saved at syscall
- * entry, do it now.
- */
-void rec_before_record_syscall_entry(Task* t, int syscallno);
-
 /**
  * Prepare |t| to enter its current syscall event.  Return ALLOW_SWITCH if
  * a context-switch is allowed for |t|, PREVENT_SWITCH if not.
@@ -29,6 +22,11 @@ Switchable rec_prepare_syscall(Task* t);
  */
 void rec_prepare_restart_syscall(Task* t);
 
+/**
+ * Inside a fork/clone syscall, notify that the new task created is new_task.
+ */
+void rec_set_syscall_new_task(Task* t, Task* new_task);
+
 /**
  * Restore any argument registers fudged for |t|'s current syscall and
  * store any nondeterministic outparam data.
