@@ -14,8 +14,16 @@ struct ReplayTraceStep;
  */
 void rep_after_enter_syscall(Task* t, int syscallno);
 
-/* Process pending syscall. Call this when |t| is about to enter or exit
- * a syscall. */
+/**
+ * Process pending syscall. Call this when |t| is about to enter or exit
+ * a syscall.
+ */
 void rep_process_syscall(Task* t, ReplayTraceStep* step);
 
+/**
+ * Process an EV_GROW_MAP event. These are like mmap syscalls, so handled
+ * in replay_syscall.
+ */
+void process_grow_map(Task* t);
+
 #endif /* RR_REP_PROCESS_EVENT_H_ */
