@@ -17,14 +17,16 @@
 #include "PerfCounters.h"
 #include "PropertyTable.h"
 #include "Registers.h"
+#include "remote_code_ptr.h"
 #include "TaskishUid.h"
 #include "TraceStream.h"
 #include "util.h"
 
 class AutoRemoteSyscalls;
-class Session;
 class RecordSession;
 class ReplaySession;
+class ScopedFd;
+class Session;
 struct Sighandlers;
 class Task;
 
@@ -39,9 +41,7 @@ struct syscallbuf_record;
  * all zeroes.
  */
 struct ReturnAddressList {
-  enum {
-    COUNT = 8
-  };
+  enum { COUNT = 8 };
   remote_ptr<void> addresses[COUNT];
 
   bool operator==(const ReturnAddressList& other) const {
@@ -150,10 +150,16 @@ enum WaitRequest {
   // Don't wait after resuming.
   RESUME_NONBLOCKING
 };
-
-enum ShareDeschedEventFd {
-  SHARE_DESCHED_EVENT_FD = 1,
-  DONT_SHARE_DESCHED_EVENT_FD = 0
+enum TicksRequest {
+  // We don't expect to see any ticks (though we seem to on the odd buggy
+  // system...). Using this is a small performance optimization because we don't
+  // have to stop and restart the performance counters. This may also avoid
+  // bugs on some systems that report performance counter advances while
+  // in the kernel...
+  RESUME_NO_TICKS = -2,
+  RESUME_UNLIMITED_TICKS = -1
+  // Positive values are a request for an interrupt
+  // after that number of ticks
 };
 
 /** Different kinds of waits a task can do.
@@ -210,36 +216,6 @@ public:
    */
   bool at_may_restart_syscall() const;
 
-  /**
-   * Continue according to the semantics implied by the helper's
-   * name.  See the ptrace manual for details of semantics.  If
-   * |sig| is nonzero, it's delivered to this as part of the
-   * resume request.
-   *
-   * By default, wait for status to change after resuming,
-   * before returning.  Return true if successful, false if
-   * interrupted.  Don't wait for status change in the
-   * "_nonblocking()" variants.
-   */
-  void cont_nonblocking(int sig = 0, Ticks tick_period = 0) {
-    resume_execution(RESUME_CONT, RESUME_NONBLOCKING, sig, tick_period);
-  }
-  void cont_singlestep(int sig = 0) {
-    resume_execution(RESUME_SINGLESTEP, RESUME_WAIT, sig);
-  }
-  void cont_syscall(int sig = 0) {
-    resume_execution(RESUME_SYSCALL, RESUME_WAIT);
-  }
-  void cont_syscall_nonblocking(int sig = 0, Ticks tick_period = 0) {
-    resume_execution(RESUME_SYSCALL, RESUME_NONBLOCKING, sig, tick_period);
-  }
-  void cont_sysemu(int sig = 0) {
-    resume_execution(RESUME_SYSEMU, RESUME_WAIT, sig);
-  }
-  void cont_sysemu_singlestep(int sig = 0) {
-    resume_execution(RESUME_SYSEMU_SINGLESTEP, RESUME_WAIT, sig);
-  }
-
   /**
    * This must be in an emulated syscall, entered through
    * |cont_sysemu()| or |cont_sysemu_singlestep()|, but that's
@@ -249,20 +225,20 @@ public:
   void finish_emulated_syscall();
 
   /**
-   * Shortcut to the single |pending_event->desched.rec| when
-   * there's one desched event on the stack, and nullptr otherwise.
+   * Shortcut to the most recent |pending_event->desched.rec| when
+   * there's a desched event on the stack, and nullptr otherwise.
    * Exists just so that clients don't need to dig around in the
    * event stack to find this record.
    */
   const struct syscallbuf_record* desched_rec() const;
 
+  /**
+   * Returns true when the task is in a signal handler in an interrupted
+   * system call being handled by syscall buffering.
+   */
+  bool running_inside_desched() const;
+
   size_t syscallbuf_data_size() const {
-    if (syscallbuf_hdr->locked) {
-      // There may be an incomplete syscall record after num_rec_bytes that
-      // we need to record. We don't know how big that record is,
-      // so just record the entire buffer. This should not be common.
-      return num_syscallbuf_bytes;
-    }
     return syscallbuf_hdr->num_rec_bytes + sizeof(*syscallbuf_hdr);
   }
 
@@ -332,12 +308,6 @@ public:
    */
   void destabilize_task_group();
 
-  /**
-   * Print out a warning message when we're in the process of delivering a
-   * fatal signal, if appropriate.
-   */
-  void notify_fatal_signal();
-
   /**
    * Emulate 'tracer' ptracing this task.
    */
@@ -419,15 +389,19 @@ public:
   Event& ev() { return pending_events.back(); }
   const Event& ev() const { return pending_events.back(); }
 
-  struct FStatResult {
-    struct stat st;
-    std::string file_name;
-  };
   /**
-   * Stat |fd| in the context of this task's fd table, returning
-   * the result and the file name in FStatResult.
+   * Stat |fd| in the context of this task's fd table.
    */
-  FStatResult fstat(int fd);
+  struct stat stat_fd(int fd);
+  /**
+   * Open |fd| in the context of this task's fd table.
+   */
+  ScopedFd open_fd(int fd, int flags);
+  /**
+   * Get the name of the file referenced by |fd| in the context of this
+   * task's fd table.
+   */
+  std::string file_name_of_fd(int fd);
 
   /**
    * Force the wait status of this to |status|, as if
@@ -439,11 +413,11 @@ public:
   void did_waitpid(int status, siginfo_t* override_siginfo = nullptr);
 
   /**
-   * If we're at the entry to (or exit from) a syscall, we may need to
-   * fix up registers that are set nondeterministically so they're
-   * deterministic.
+   * Syscalls have side effects on registers (e.g. setting the flags register).
+   * Perform those side effects on |regs| and do set_regs() on that to make it
+   * look like a syscall happened.
    */
-  void fixup_syscall_regs();
+  void emulate_syscall_entry(const Registers& regs);
 
   /**
    * Wait for |futex| in this address space to have the value
@@ -491,8 +465,7 @@ public:
    *
    * Pass SHARE_DESCHED_EVENT_FD to additionally share that fd.
    */
-  void init_buffers(remote_ptr<void> map_hint,
-                    ShareDeschedEventFd share_desched_fd);
+  void init_buffers(remote_ptr<void> map_hint);
 
   /**
    * Destroy in the tracee task the scratch buffer and syscallbuf (if
@@ -503,7 +476,7 @@ public:
   void destroy_buffers();
 
   /** Return the current $ip of this. */
-  remote_ptr<uint8_t> ip() { return regs().ip(); }
+  remote_code_ptr ip() { return regs().ip(); }
 
   /**
    * Return true if this is at an arm-desched-event syscall.
@@ -527,7 +500,10 @@ public:
    * code. Callers may assume |is_in_syscallbuf()| is implied
    * by this.
    */
-  bool is_entering_traced_syscall() { return ip() == as->traced_syscall_ip(); }
+  bool is_entering_traced_syscall() {
+    return ip() == as->traced_syscall_ip() ||
+           ip() == as->privileged_traced_syscall_ip();
+  }
 
   /**
    * Return true if this is within the syscallbuf library.  This
@@ -535,7 +511,7 @@ public:
    * below.
    */
   bool is_in_syscallbuf() {
-    remote_ptr<void> p = ip();
+    remote_ptr<void> p = ip().to_data_ptr<void>();
     return (as->syscallbuf_lib_start() <= p && p < as->syscallbuf_lib_end()) ||
            (as->rr_page_start() <= p && p < as->rr_page_end());
   }
@@ -548,7 +524,11 @@ public:
    */
   bool is_in_traced_syscall() {
     return ip() ==
-           as->traced_syscall_ip() + rr::syscall_instruction_length(arch());
+               as->traced_syscall_ip().increment_by_syscall_insn_length(
+                   arch()) ||
+           ip() ==
+               as->privileged_traced_syscall_ip()
+                   .increment_by_syscall_insn_length(arch());
   }
 
   /**
@@ -559,21 +539,11 @@ public:
    * instruction.
    */
   bool is_in_untraced_syscall() {
-    return ip() ==
-           as->untraced_syscall_ip() + rr::syscall_instruction_length(arch());
+    return ip() == AddressSpace::rr_page_ip_in_untraced_syscall() ||
+           ip() == AddressSpace::rr_page_ip_in_untraced_replayed_syscall() ||
+           ip() == AddressSpace::rr_page_ip_in_privileged_untraced_syscall();
   }
 
-  /**
-   * Return true if this task is most likely entering or exiting
-   * a syscall.
-   *
-   * No false negatives are known to be possible (i.e. if this
-   * task really is replaying a syscall, true will be returned),
-   * however false positives are possible.  Callers should
-   * therefore use this return value conservatively.
-   */
-  bool is_probably_replaying_syscall();
-
   /**
    * Return true if |ptrace_event()| is the trace event
    * generated by the syscallbuf seccomp-bpf when a traced
@@ -584,6 +554,9 @@ public:
   /** Return true iff |sig| is blocked for this. */
   bool is_sig_blocked(int sig) const;
 
+  /** Set |sig| to be treated as blocked. */
+  void set_sig_blocked(int sig);
+
   /**
    * Return true iff |sig| is SIG_IGN, or it's SIG_DFL and the
    * default disposition is "ignore".
@@ -624,6 +597,19 @@ public:
    */
   void move_ip_before_breakpoint();
 
+  /**
+   * Assuming we've just entered a syscall, exit that syscall and reset
+   * state to reenter the syscall just as it was called the first time.
+   */
+  void exit_syscall_and_prepare_restart();
+
+  /**
+   * Resume execution until we get a syscall entry or exit event.
+   * During recording, any signals received are stashed.
+   * seccomp events are ignored; we assume this syscall is under rr's control.
+   */
+  void advance_syscall();
+
   /**
    * Return the "task name"; i.e. what |prctl(PR_GET_NAME)| or
    * /proc/tid/comm would say that the task's name is.
@@ -638,7 +624,8 @@ public:
    * recorded immediately after the exec.
    */
   void post_exec(const Registers* replay_regs = nullptr,
-                 const ExtraRegisters* replay_extra_regs = nullptr);
+                 const ExtraRegisters* replay_extra_regs = nullptr,
+                 const std::string* replay_exe = nullptr);
 
   /**
    * Call this method when this task has exited a successful execve() syscall.
@@ -668,7 +655,7 @@ public:
    * return.
    */
   template <size_t N>
-  void read_bytes(remote_ptr<void> child_addr, uint8_t (&buf)[N]) {
+  void read_bytes(remote_ptr<void> child_addr, uint8_t(&buf)[N]) {
     return read_bytes_helper(child_addr, N, buf);
   }
 
@@ -681,7 +668,14 @@ public:
    * |record_event()| records the specified event.
    */
   void record_current_event();
-  void record_event(const Event& ev);
+  enum FlushSyscallbuf {
+    FLUSH_SYSCALLBUF,
+    /* Pass this if it's safe to replay the event before we process the
+     * syscallbuf records.
+     */
+    DONT_FLUSH_SYSCALLBUF
+  };
+  void record_event(const Event& ev, FlushSyscallbuf flush = FLUSH_SYSCALLBUF);
 
   /**
    * Save tracee data to the trace.  |addr| is the address in
@@ -701,6 +695,9 @@ public:
     record_remote(addr, sizeof(T));
   }
 
+  // Record as much as we can of the bytes in this range.
+  void record_remote_fallible(remote_ptr<void> addr, ssize_t num_bytes);
+
   /**
    * Save tracee data to the trace.  |addr| is the address in
    * the address space of this task.
@@ -821,8 +818,8 @@ public:
    * You probably want to use one of the cont*() helpers above,
    * and not this.
    */
-  void resume_execution(ResumeRequest how, WaitRequest wait_how, int sig = 0,
-                        Ticks tick_period = 0);
+  void resume_execution(ResumeRequest how, WaitRequest wait_how,
+                        TicksRequest tick_period, int sig = 0);
 
   /** Return the session this is part of. */
   Session& session() const { return *session_; }
@@ -877,11 +874,13 @@ public:
   size_t robust_list_len() const { return robust_futex_list_len; }
 
   /** Update the thread area to |addr|. */
-  void set_thread_area(remote_ptr<void> tls);
-  const struct user_desc* tls() const;
+  void set_thread_area(remote_ptr<struct user_desc> tls);
+
+  const std::vector<struct user_desc>& thread_areas() { return thread_areas_; }
 
   /** Update the clear-tid futex to |tid_addr|. */
   void set_tid_addr(remote_ptr<int> tid_addr);
+  remote_ptr<int> tid_addr() { return tid_futex; }
 
   /**
    * Call this after |sig| is delivered to this task.  Emulate
@@ -902,7 +901,12 @@ public:
    * If signal_has_user_handler(sig) is true, return the address of the
    * user handler, otherwise return null.
    */
-  remote_ptr<uint8_t> get_signal_user_handler(int sig) const;
+  remote_code_ptr get_signal_user_handler(int sig) const;
+  /**
+   * Return true if the signal handler for |sig| takes a siginfo_t*
+   * parameter.
+   */
+  bool signal_handler_takes_siginfo(int sig) const;
 
   /**
    * Return |sig|'s current sigaction. Returned as raw bytes since the
@@ -931,9 +935,23 @@ public:
    * stash anything.
    */
   void stash_sig();
+  void stash_synthetic_sig(const siginfo_t& si);
   bool has_stashed_sig() const { return !stashed_signals.empty(); }
   siginfo_t peek_stash_sig();
-  siginfo_t pop_stash_sig();
+  void pop_stash_sig();
+
+  /**
+   * When a signal triggers an emulated a ptrace-stop for this task,
+   * save the siginfo so a later emulated ptrace-continue with this signal
+   * number can use it.
+   */
+  void save_ptrace_signal_siginfo(const siginfo_t& si);
+  /**
+   * When emulating a ptrace-continue with a signal number, extract the siginfo
+   * that was saved by |save_ptrace_signal_siginfo|. If no such siginfo was
+   * saved, make one up.
+   */
+  siginfo_t take_ptrace_signal_siginfo(int sig);
 
   /**
    * Return true when the task is running, false if it's stopped.
@@ -1011,9 +1029,32 @@ public:
    * Call this before recording events or data.  Records
    * syscallbuf data and flushes the buffer, if there's buffered
    * data.
+   *
+   * The timing of calls to this is tricky. We must flush the syscallbuf
+   * before recording any data associated with events that happened after the
+   * buffered syscalls. But we don't support flushing a syscallbuf twice with
+   * no intervening reset, i.e. after flushing we have to be sure we'll get
+   * a chance to reset the syscallbuf (i.e. record some other kind of event)
+   * before the tracee runs again in a way that might append another buffered
+   * syscall --- so we can't flush too early
    */
   void maybe_flush_syscallbuf();
 
+  /**
+   * Call this after recording an event when it might be safe to reset the
+   * syscallbuf. It must be after recording an event to ensure during replay
+   * we run past any syscallbuf after-syscall code that uses the buffer data.
+   */
+  void maybe_reset_syscallbuf();
+
+  /**
+   * Call this to reset syscallbuf_hdr->num_rec_bytes and zero out the data
+   * recorded in the syscall buffer. This makes for more deterministic behavior
+   * especially during replay, where during checkpointing we only save and
+   * restore the recorded data area.
+   */
+  void reset_syscallbuf();
+
   /**
    * Return the virtual memory mapping (address space) of this
    * task.
@@ -1049,16 +1090,18 @@ public:
    * Write |N| bytes from |buf| to |child_addr|, or don't return.
    */
   template <size_t N>
-  void write_bytes(remote_ptr<void> child_addr, const uint8_t (&buf)[N]) {
+  void write_bytes(remote_ptr<void> child_addr, const uint8_t(&buf)[N]) {
     write_bytes_helper(child_addr, N, buf);
   }
 
   /**
    * Write |val| to |child_addr|.
    */
-  template <typename T> void write_mem(remote_ptr<T> child_addr, const T& val) {
+  template <typename T>
+  void write_mem(remote_ptr<T> child_addr, const T& val, bool* ok = nullptr) {
     assert(type_has_no_holes<T>());
-    write_bytes_helper(child_addr, sizeof(val), static_cast<const void*>(&val));
+    write_bytes_helper(child_addr, sizeof(val), static_cast<const void*>(&val),
+                       ok);
   }
   /**
    * This is not the helper you're looking for.  See above: you
@@ -1091,7 +1134,7 @@ public:
   void read_bytes_helper(remote_ptr<void> addr, ssize_t buf_size, void* buf,
                          bool* ok = nullptr);
   void write_bytes_helper(remote_ptr<void> addr, ssize_t buf_size,
-                          const void* buf);
+                          const void* buf, bool* ok = nullptr);
 
   /** See |pending_sig()| above. */
   int pending_sig_from_status(int status) const;
@@ -1248,17 +1291,12 @@ public:
    * next available slow (taking |desched| into
    * consideration). */
   bool flushed_syscallbuf;
+  /* Value of hdr->num_rec_bytes when the buffer was flushed */
+  uint32_t flushed_num_rec_bytes;
   /* This bit is set when code wants to prevent the syscall
    * record buffer from being reset when it normally would be.
    * Currently, the desched'd syscall code uses this. */
   bool delay_syscallbuf_reset;
-  /* This bit is set when code wants the syscallbuf to be
-   * "synthetically empty": even if the record counter is
-   * nonzero, it should not be flushed.  Currently, the
-   * desched'd syscall code uses this along with
-   * |delay_syscallbuf_reset| above to keep the syscallbuf
-   * intact during possibly many "reentrant" events. */
-  bool delay_syscallbuf_flush;
 
   /* The child's desched counter event fd number, and our local
    * dup. */
@@ -1274,14 +1312,8 @@ public:
    * any untraced ones; that's the magic "rrcall" the tracee
    * uses to initialize its syscallbuf. */
   bool seccomp_bpf_enabled;
-
-  /* State used only during replay. */
-
-  int child_sig;
-  // True when this has been forced to enter a syscall with
-  // PTRACE_SYSCALL when instead we wanted to use
-  // PTRACE_SINGLESTEP.  See replayer.cc.
-  bool stepped_into_syscall;
+  // Value to return from PR_GET_SECCOMP
+  uint8_t prctl_seccomp_status;
 
   /* State used during both recording and replay. */
 
@@ -1293,6 +1325,10 @@ public:
    * recording, it's synonymous with |tid|, and during replay
    * it's the tid that was recorded. */
   pid_t rec_tid;
+  /* This is the recorded tid of the tracee *in its own pid namespace*.
+   * Only valid during recording, otherwise 0!
+   */
+  pid_t own_namespace_rec_tid;
 
   /* Points at rr's mapping of the (shared) syscall buffer. */
   struct syscallbuf_hdr* syscallbuf_hdr;
@@ -1300,6 +1336,8 @@ public:
   /* Points at the tracee's mapping of the buffer. */
   remote_ptr<struct syscallbuf_hdr> syscallbuf_child;
   remote_ptr<char> syscallbuf_fds_disabled_child;
+  remote_code_ptr stopping_breakpoint_table;
+  int stopping_breakpoint_table_entry_size;
 
   PropertyTable& properties() { return properties_; }
 
@@ -1311,8 +1349,7 @@ public:
     std::string prname;
     remote_ptr<void> robust_futex_list;
     size_t robust_futex_list_len;
-    struct user_desc thread_area;
-    bool thread_area_valid;
+    std::vector<struct user_desc> thread_areas;
     size_t num_syscallbuf_bytes;
     int desched_fd_child;
     remote_ptr<struct syscallbuf_hdr> syscallbuf_child;
@@ -1339,9 +1376,7 @@ private:
   template <typename Arch> void update_sigaction_arch(const Registers& regs);
 
   /** Helper function for init_buffers. */
-  template <typename Arch>
-  void init_buffers_arch(remote_ptr<void> map_hint,
-                         ShareDeschedEventFd share_desched_fd);
+  template <typename Arch> void init_buffers_arch(remote_ptr<void> map_hint);
 
   /**
    * Return a new Task cloned from |p|.  |flags| are a set of
@@ -1528,17 +1563,15 @@ private:
   std::shared_ptr<Sighandlers> sighandlers;
   // Stashed signal-delivery state, ready to be delivered at
   // next opportunity.
-  struct StashedSignal {
-    StashedSignal(const siginfo_t& si) : si(si) {}
-    siginfo_t si;
-  };
-  std::deque<StashedSignal> stashed_signals;
+  std::deque<siginfo_t> stashed_signals;
+  // Saved emulated-ptrace signals
+  std::vector<siginfo_t> saved_ptrace_siginfos;
   // The task group this belongs to.
   std::shared_ptr<TaskGroup> tg;
-  // Contents of the |tls| argument passed to |clone()| and
-  // |set_thread_area()|, when |thread_area_valid| is true.
-  struct user_desc thread_area;
-  bool thread_area_valid;
+  // Entries set by |set_thread_area()| or the |tls| argument to |clone()|
+  // (when that's a user_desc). May be more than one due to different
+  // entry_numbers.
+  std::vector<struct user_desc> thread_areas_;
   // The memory cell the kernel will clear and notify on exit,
   // if our clone parent requested it.
   remote_ptr<int> tid_futex;
