@@ -2,7 +2,7 @@
 
 #include "rrutil.h"
 
-int main(int argc, char* argv[]) {
+int main(void) {
   /* Fork-and-exec 'echo'.
      The exec may fail if 'bash' is 64-bit and rr doesn't support
      64-bit processes. That's fine; the test should still pass. We're
