@@ -2,14 +2,14 @@
 
 #include "rrutil.h"
 
-static void* kill_thread(void* dontcare) {
+static void* kill_thread(__attribute__((unused)) void* dontcare) {
   atomic_puts("killing ...");
   abort();
   atomic_puts("FAILED: abort() didn't work");
   return NULL; /* not reached */
 }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   pthread_t t;
 
   pthread_create(&t, NULL, kill_thread, NULL);
