@@ -4,13 +4,15 @@
 
 static void breakpoint(void) {}
 
-int caught_sig = 0;
+static volatile int caught_sig = 0;
 
-void catcher(int signum, siginfo_t* siginfo_ptr, void* ucontext_ptr) {
+void catcher(__attribute__((unused)) int signum,
+             __attribute__((unused)) siginfo_t* siginfo_ptr,
+             __attribute__((unused)) void* ucontext_ptr) {
   caught_sig = signum;
 }
 
-int main(int argc, char** argv) {
+int main(void) {
   struct sigaction sact;
   int counter;
 
