@@ -2,7 +2,7 @@
 
 #include "rrutil.h"
 
-int main(int argc, char* argv[]) {
+int main(void) {
   signal(SIGSEGV, SIG_IGN);
 
   kill(getpid(), SIGSEGV);
