@@ -4,15 +4,17 @@
 
 static pthread_key_t exit_key;
 
-static void thread_exit(void* data) { atomic_puts("thread exit"); }
+static void thread_exit(__attribute__((unused)) void* data) {
+  atomic_puts("thread exit");
+}
 
-static void* thread(void* unused) {
+static void* thread(__attribute__((unused)) void* unused) {
   pthread_key_create(&exit_key, thread_exit);
   pthread_setspecific(exit_key, (void*)0x1);
   pthread_exit(NULL);
 }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   pthread_t t;
 
   pthread_create(&t, NULL, thread, NULL);
