@@ -25,12 +25,29 @@
 #define STATEMENT64(i) STATEMENT32(i) STATEMENT32(i + 32)
 #define STATEMENT128(i) STATEMENT64(i) STATEMENT64(i + 64)
 #define STATEMENT256(i) STATEMENT128(i) STATEMENT128(i + 128)
+#define STATEMENT512(i) STATEMENT256(i) STATEMENT256(i + 256)
+#define STATEMENT1024(i) STATEMENT512(i) STATEMENT512(i + 512)
+#define STATEMENT2048(i) STATEMENT1024(i) STATEMENT1024(i + 1024)
+#define STATEMENT4096(i) STATEMENT2048(i) STATEMENT2048(i + 2048)
+
+static void* do_thread(void* p) {
+  while (1) {
+    sched_yield();
+  }
+  return NULL;
+}
 
 int main(int argc, char** argv) {
   int a = atoi(argv[1]);
   int b = atoi(argv[2]);
+  pthread_t thread;
+
+  /* Create an extra thread so context switches can happen
+     and SCHED events will be recorded. */
+  pthread_create(&thread, NULL, do_thread, NULL);
+
   /* This syscall signals the test that we're in the test body proper */
   getgid();
-  STATEMENT256(0)
+  STATEMENT4096(0)
   return a + b;
 }
