@@ -1,12 +1,12 @@
 from rrutil import *
 
-send_gdb('b main\n')
+send_gdb('b main')
 expect_gdb('Breakpoint 1')
 
-send_gdb('c\n')
+send_gdb('c')
 expect_gdb('Breakpoint 1, main')
 
-send_gdb('call exit(0)\n')
+send_gdb('call exit(0)')
 expect_gdb('while in a function called from GDB')
 
 restart_replay()
