@@ -3,10 +3,10 @@ import re
 
 NUM_THREADS = 10
 
-send_gdb('b hit_barrier\n')
+send_gdb('b hit_barrier')
 expect_gdb('Breakpoint 1')
 
-send_gdb('c\n')
+send_gdb('c')
 expect_gdb('Breakpoint 1, hit_barrier')
 
 arch = get_exe_arch()
@@ -14,11 +14,12 @@ arch = get_exe_arch()
 # The locations the threads are stopped at depends on the architecture.
 stopped_locations = {
     # on i386, we sometimes stop in the middle of nowhere
-    'i386': ['(0x[0-9a-f]+ in )?__kernel_vsyscall', '(0x[0-9a-f]+ in )?_traced_raw_syscall', '0x[0-9a-f]+ in \?\?'],
+    'i386': ['(0x[0-9a-f]+ in )?__kernel_vsyscall', '(0x[0-9a-f]+ in )?_traced_raw_syscall', '0x[0-9a-f]+ in \?\?',
+             '(0x[0-9a-f]+ in )?__lll_lock_wait', '(0x[0-9a-f]+ in )?pthread_barrier_wait'],
     'i386:x86-64': ['(0x[0-9a-f]+ in )?__lll_lock_wait', '(0x[0-9a-f]+ in )?pthread_barrier_wait', '0x70000010 in \?\?'],
 }
 
-send_gdb('info threads\n')
+send_gdb('info threads')
 for i in xrange(NUM_THREADS + 1, 1, -1):
     # The threads are at the kernel syscall entry, or either the
     # traced/untraced entry reached through the rr monkeypatched one.
