@@ -18,11 +18,11 @@ static void* child_thread(void* num_syscallsp) {
 
   first_breakpoint();
 
-  /* NB: this test assumes that gettid() produces at least one
+  /* NB: this test assumes that geteuid() produces at least one
    * trace event per syscall. */
   atomic_printf("%d: running %d syscalls ...\n", getpid(), num_syscalls);
   for (i = 0; i < num_syscalls; ++i) {
-    (void)sys_gettid();
+    geteuid();
   }
 
   second_breakpoint();
