@@ -3,7 +3,7 @@
 #include "rrutil.h"
 
 int main(int argc, char* argv[]) {
-  int dummy, i;
+  int dummy = 0, i;
 
   /* NB: since we're masking out the signal, there's no way for
    * us to tell whether or not it was actually delivered.  This
