@@ -4,7 +4,7 @@
 
 int main(int argc, char* argv[]) {
   size_t page_size = sysconf(_SC_PAGESIZE);
-  uint8_t* map1 = mmap(NULL, page_size, PROT_READ | PROT_WRITE,
+  uint8_t* map1 = mmap(NULL, page_size * 2, PROT_READ | PROT_WRITE,
                        MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
   uint8_t* map1_end = map1 + page_size;
   uint8_t* map2;
