@@ -2,6 +2,8 @@
 
 #include "rrutil.h"
 
+#define NEW_VALUE 0xabcdef
+
 static size_t static_data = 0x12345678;
 
 int main(int argc, char* argv[]) {
@@ -13,17 +15,16 @@ int main(int argc, char* argv[]) {
   struct user_fpxregs_struct* fpxregs;
 #endif
   int pipe_fds[2];
-  struct timespec ts = { 0, 50000000 };
 
   test_assert(0 == pipe(pipe_fds));
 
   if (0 == (child = fork())) {
     char ch;
     read(pipe_fds[0], &ch, 1);
+    test_assert(static_data == NEW_VALUE);
     return 77;
   }
 
-  nanosleep(&ts, NULL);
   test_assert(0 == ptrace(PTRACE_ATTACH, child, NULL, NULL));
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == ((SIGSTOP << 8) | 0x7f));
@@ -55,6 +56,15 @@ int main(int argc, char* argv[]) {
 
   test_assert(static_data ==
               ptrace(PTRACE_PEEKDATA, child, &static_data, NULL));
+  test_assert(0 ==
+              ptrace(PTRACE_POKEDATA, child, &static_data, (void*)NEW_VALUE));
+  test_assert(NEW_VALUE == ptrace(PTRACE_PEEKDATA, child, &static_data, NULL));
+
+  /* Test invalid locations */
+  test_assert(-1 == ptrace(PTRACE_PEEKDATA, child, NULL, NULL));
+  test_assert(errno == EIO || errno == EFAULT);
+  test_assert(-1 == ptrace(PTRACE_POKEDATA, child, NULL, (void*)NEW_VALUE));
+  test_assert(errno == EIO || errno == EFAULT);
 
   test_assert(regs->eflags == ptrace(PTRACE_PEEKUSER, child,
                                      (void*)offsetof(struct user, regs.eflags),
