@@ -1,15 +1,15 @@
 from rrutil import *
 
-send_gdb('b breakpoint\n')
+send_gdb('b breakpoint')
 expect_gdb('Breakpoint 1')
 
-send_gdb('c\n')
+send_gdb('c')
 expect_gdb('Breakpoint 1, breakpoint')
 
-send_gdb('f 1\n')
-expect_gdb('(gdb)')
+send_gdb('f 1')
+expect_gdb('(rr)')
 
-send_gdb('p big\n')
+send_gdb('p big')
 expect_gdb("bytes = 'Z'")
 
 ok()
