@@ -7,11 +7,14 @@
 #define _POSIX_C_SOURCE 2
 
 #include <arpa/inet.h>
+#include <asm/prctl.h>
 #include <assert.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <linux/audit.h>
+#include <linux/capability.h>
 #include <linux/ethtool.h>
 #include <linux/filter.h>
 #include <linux/futex.h>
@@ -21,6 +24,7 @@
 #include <linux/random.h>
 #include <linux/seccomp.h>
 #include <linux/sockios.h>
+#include <linux/unistd.h>
 #include <linux/videodev2.h>
 #include <linux/wireless.h>
 #include <poll.h>
@@ -28,6 +32,7 @@
 #include <pty.h>
 #include <sched.h>
 #include <signal.h>
+#include <sound/asound.h>
 #include <stdarg.h>
 #include <stddef.h>
 #include <stdint.h>
@@ -35,6 +40,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syscall.h>
+#include <sys/file.h>
 #include <sys/xattr.h>
 #include <sys/epoll.h>
 #include <sys/ioctl.h>
@@ -58,6 +64,7 @@
 #include <sys/times.h>
 #include <sys/timerfd.h>
 #include <sys/types.h>
+#include <sys/ucontext.h>
 #include <sys/uio.h>
 #include <sys/un.h>
 #include <sys/user.h>
@@ -66,6 +73,7 @@
 #include <sys/wait.h>
 #include <termios.h>
 #include <time.h>
+#include <ucontext.h>
 #include <unistd.h>
 #include <utime.h>
 #include <x86intrin.h>
@@ -89,9 +97,8 @@ typedef unsigned char uint8_t;
  * manage.  Async-signal-safe.  Does not flush stdio buffers (doing so
  * isn't signal safe).
  */
-__attribute__((format(printf, 1,
-                      2))) inline static int atomic_printf(const char* fmt,
-                                                           ...) {
+__attribute__((format(printf, 1, 2))) inline static int atomic_printf(
+    const char* fmt, ...) {
   va_list args;
   char buf[1024];
   int len;
