@@ -1,6 +1,6 @@
 from rrutil import *
 
-send_gdb('c\n')
+send_gdb('c')
 expect_rr('Hi')
 
 ok()
