@@ -16,12 +16,23 @@ static void handle_sigrt(int sig) {
 }
 
 static void my_raise(int sig) {
+/* Don't call raise() directly, since that can go through our syscall hooks
+   which mess up gdb's reverse-finish slightly. */
 #ifdef __i386__
-  /* Don't call raise() directly, since that can go through our syscall hooks
-     which mess up gdb's reverse-finish slightly. */
   int tid = getpid();
-  __asm__ __volatile__("int $0x80\n\t" ::"a"(SYS_tgkill), "b"(tid), "c"(tid),
-                       "d"(sig));
+  /* Use a special instruction after the syscall to make sure we don't patch
+     it */
+  __asm__ __volatile__("xchg %%ebx,%%edi\n\t"
+                       "int $0x80\n\t"
+                       "xchg %%ebx,%%edi\n\t" ::"a"(SYS_tgkill),
+                       "c"(tid), "d"(sig), "D"(tid));
+#elif defined(__x86_64__)
+  int tid = getpid();
+  /* Use a special instruction after the syscall to make sure we don't patch
+     it */
+  __asm__ __volatile__("syscall\n\t"
+                       "xchg %%rdx,%%rdx\n\t" ::"a"(SYS_tgkill),
+                       "D"(tid), "S"(tid), "d"(sig));
 #else
   raise(sig);
 #endif
