@@ -2,7 +2,7 @@
 
 #include "rrutil.h"
 
-int main(int argc, char* argv[]) {
+int main(void) {
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
