@@ -4,14 +4,17 @@
 
 int main(int argc, char** argv) {
   int fd;
-  char buf[1 << 12];
+  char buf[10];
   int i;
 
   fd = open("/dev/zero", O_RDONLY);
-  for (i = 0; i < 1 << 8; ++i) {
+  for (i = 0; i < 1 << 12; ++i) {
     read(fd, buf, sizeof(buf));
+    if (!(i & ((1 << 8) - 1))) {
+      atomic_printf(".");
+    }
   }
 
-  atomic_puts("EXIT-SUCCESS");
+  atomic_puts("\nEXIT-SUCCESS");
   return 0;
 }
