@@ -70,7 +70,7 @@ function onexit {
     if [[ "$leave_data" != "y" ]]; then
         rm -rf $workdir
     else
-        echo Test $TESTNAME failed, leaving behind $workdir.
+        echo Test $TESTNAME failed, leaving behind $workdir
         echo To replay the failed test, run
         echo " " _RR_TRACE_DIR="$workdir" rr replay
     fi
@@ -148,6 +148,9 @@ if [[ ! -d $TESTDIR ]]; then
     fatal FAILED: TESTDIR "($TESTDIR)" not found.
 fi
 
+# Our test programs intentionally crash a lot. Don't generate coredumps for them.
+ulimit -c 0
+
 # NB: must set up the trap handler *before* mktemp
 trap onexit EXIT
 workdir=`mktemp -dt rr-test-$TESTNAME-XXXXXXXXX`
