@@ -13,10 +13,9 @@
 #include <elf.h>
 #include <fcntl.h>
 #include <inttypes.h>
-#include <linux/filter.h>
+#include <limits.h>
 #include <linux/magic.h>
 #include <linux/prctl.h>
-#include <linux/seccomp.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/vfs.h>
@@ -24,12 +23,14 @@
 
 #include "preload/preload_interface.h"
 
+#include "AddressSpace.h"
 #include "AutoRemoteSyscalls.h"
 #include "Flags.h"
 #include "kernel_abi.h"
 #include "kernel_metadata.h"
 #include "log.h"
 #include "ReplaySession.h"
+#include "seccomp-bpf.h"
 #include "task.h"
 #include "TraceStream.h"
 
@@ -151,21 +152,18 @@ void dump_process_memory(Task* t, TraceFrame::Time global_time,
   dump_file = fopen64(filename, "w");
 
   const AddressSpace& as = *(t->vm());
-  for (auto& kv : as.memmap()) {
-    const Mapping& first = kv.first;
-    const MappableResource& second = kv.second;
+  for (auto m : as.maps()) {
     vector<uint8_t> mem;
-    mem.resize(first.num_bytes());
+    mem.resize(m.map.size());
 
     ssize_t mem_len =
-        t->read_bytes_fallible(first.start, first.num_bytes(), mem.data());
+        t->read_bytes_fallible(m.map.start(), m.map.size(), mem.data());
     mem_len = max(ssize_t(0), mem_len);
 
-    string label = first.str() + ' ' + second.str();
-
-    if (!is_start_of_scratch_region(t, first.start)) {
-      dump_binary_chunk(dump_file, label.c_str(), (const uint32_t*)mem.data(),
-                        mem_len / sizeof(uint32_t), first.start);
+    if (!is_start_of_scratch_region(t, m.map.start())) {
+      dump_binary_chunk(dump_file, m.map.str().c_str(),
+                        (const uint32_t*)mem.data(), mem_len / sizeof(uint32_t),
+                        m.map.start());
     }
   }
   fclose(dump_file);
@@ -214,35 +212,31 @@ static void notify_checksum_error(Task* t, TraceFrame::Time global_time,
  * checksums.  The iterator data determines which behavior the helper
  * function takes on, and to/from which file it writes/read.
  */
-enum ChecksumMode {
-  STORE_CHECKSUMS,
-  VALIDATE_CHECKSUMS
-};
+enum ChecksumMode { STORE_CHECKSUMS, VALIDATE_CHECKSUMS };
 struct checksum_iterator_data {
   ChecksumMode mode;
   FILE* checksums_file;
   TraceFrame::Time global_time;
 };
 
-static bool checksum_segment_filter(const Mapping& m,
-                                    const MappableResource& r) {
+static bool checksum_segment_filter(const AddressSpace::Mapping& m) {
   struct stat st;
   int may_diverge;
 
-  if (stat(r.fsname.c_str(), &st)) {
+  if (stat(m.map.fsname().c_str(), &st)) {
     /* If there's no persistent resource backing this
      * mapping, we should expect it to change. */
-    LOG(debug) << "CHECKSUMMING unlinked '" << r.fsname << "'";
+    LOG(debug) << "CHECKSUMMING unlinked '" << m.map.fsname() << "'";
     return true;
   }
   /* If we're pretty sure the backing resource is effectively
    * immutable, skip checksumming, it's a waste of time.  Except
    * if the mapping is mutable, for example the rw data segment
    * of a system library, then it's interesting. */
-  may_diverge = should_copy_mmap_region(r.fsname, &st, m.prot, m.flags) ||
-                (PROT_WRITE & m.prot);
+  may_diverge =
+      should_copy_mmap_region(m.map, st) || (PROT_WRITE & m.map.prot());
   LOG(debug) << (may_diverge ? "CHECKSUMMING" : "  skipping") << " '"
-             << r.fsname << "'";
+             << m.map.fsname() << "'";
   return may_diverge;
 }
 
@@ -268,17 +262,14 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
   }
 
   const AddressSpace& as = *(t->vm());
-  for (auto& kv : as.memmap()) {
-    const Mapping& first = kv.first;
-    const MappableResource& second = kv.second;
-
+  for (auto m : as.maps()) {
     vector<uint8_t> mem;
     ssize_t valid_mem_len = 0;
 
-    if (checksum_segment_filter(first, second)) {
-      mem.resize(first.num_bytes());
+    if (checksum_segment_filter(m)) {
+      mem.resize(m.map.size());
       valid_mem_len =
-          t->read_bytes_fallible(first.start, first.num_bytes(), mem.data());
+          t->read_bytes_fallible(m.map.start(), m.map.size(), mem.data());
       valid_mem_len = max(ssize_t(0), valid_mem_len);
     }
 
@@ -286,7 +277,7 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
     unsigned checksum = 0;
     int i;
 
-    if (second.fsname.find(SYSCALLBUF_SHMEM_PATH_PREFIX) != string::npos) {
+    if (m.map.fsname().find(SYSCALLBUF_SHMEM_PATH_PREFIX) == 0) {
       /* The syscallbuf consists of a region that's written
       * deterministically wrt the trace events, and a
       * region that's written nondeterministically in the
@@ -299,7 +290,7 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
       *
       * So here, we set things up so that we only checksum
       * the deterministic region. */
-      auto child_hdr = first.start.cast<struct syscallbuf_hdr>();
+      auto child_hdr = m.map.start().cast<struct syscallbuf_hdr>();
       auto hdr = t->read_mem(child_hdr);
       valid_mem_len = !buf ? 0 : sizeof(hdr) + hdr.num_rec_bytes +
                                      sizeof(struct syscallbuf_record);
@@ -310,7 +301,7 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
       checksum += buf[i];
     }
 
-    string raw_map_line = first.str() + ' ' + second.str();
+    string raw_map_line = m.map.str();
     if (STORE_CHECKSUMS == c.mode) {
       fprintf(c.checksums_file, "(%x) %s\n", checksum, raw_map_line.c_str());
     } else {
@@ -327,9 +318,9 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
       remote_ptr<void> rec_end_addr = rec_end;
       ASSERT(t, 3 == nparsed) << "Only parsed " << nparsed << " items";
 
-      ASSERT(t, rec_start_addr == first.start && rec_end_addr == first.end)
+      ASSERT(t, rec_start_addr == m.map.start() && rec_end_addr == m.map.end())
           << "Segment " << rec_start_addr << "-" << rec_end_addr
-          << " changed to " << first << "??";
+          << " changed to " << m.map << "??";
 
       if (is_start_of_scratch_region(t, rec_start_addr)) {
         /* Replay doesn't touch scratch regions, so
@@ -387,7 +378,7 @@ void validate_process_memory(Task* t, TraceFrame::Time global_time) {
 }
 
 signal_action default_action(int sig) {
-  if (SIGRTMIN <= sig && sig <= SIGRTMAX) {
+  if (32 <= sig && sig <= 64) {
     return TERMINATE;
   }
   switch (sig) {
@@ -463,27 +454,6 @@ SignalDeterministic is_deterministic_signal(const siginfo_t& si) {
   }
 }
 
-bool possibly_destabilizing_signal(Task* t, int sig,
-                                   SignalDeterministic deterministic) {
-  signal_action action = default_action(sig);
-  if (action != DUMP_CORE && action != TERMINATE) {
-    // If the default action doesn't kill the process, it won't die.
-    return false;
-  }
-
-  if (t->is_sig_ignored(sig)) {
-    // Deterministic fatal signals can't be ignored.
-    return deterministic == DETERMINISTIC_SIG;
-  }
-  if (!t->signal_has_user_handler(sig)) {
-    // The default action is going to happen: killing the process.
-    return true;
-  }
-  // If the signal's blocked, user handlers aren't going to run and the process
-  // will die.
-  return t->is_sig_blocked(sig);
-}
-
 static bool has_fs_name(const string& path) {
   struct stat dummy;
   return 0 == stat(path.c_str(), &dummy);
@@ -493,14 +463,16 @@ static bool is_tmp_file(const string& path) {
   struct statfs sfs;
   statfs(path.c_str(), &sfs);
   return (TMPFS_MAGIC == sfs.f_type
-                         // In observed configurations of Ubuntu 13.10, /tmp is
-                         // a folder in the / fs, not a separate tmpfs.
-          ||
-          path.c_str() == strstr(path.c_str(), "/tmp/"));
+          // In observed configurations of Ubuntu 13.10, /tmp is
+          // a folder in the / fs, not a separate tmpfs.
+          || path.c_str() == strstr(path.c_str(), "/tmp/"));
 }
 
-bool should_copy_mmap_region(const string& file_name, const struct stat* stat,
-                             int prot, int flags) {
+bool should_copy_mmap_region(const KernelMapping& mapping,
+                             const struct stat& stat) {
+  int flags = mapping.flags();
+  int prot = mapping.prot();
+  const string& file_name = mapping.fsname();
   bool private_mapping = (flags & MAP_PRIVATE);
 
   // TODO: handle mmap'd files that are unlinked during
@@ -513,16 +485,17 @@ bool should_copy_mmap_region(const string& file_name, const struct stat* stat,
     LOG(debug) << "  copying file on tmpfs";
     return true;
   }
+  if (file_name == "/etc/ld.so.cache") {
+    // This file changes on almost every system update so we should copy it.
+    LOG(debug) << "  copying " << file_name;
+    return true;
+  }
   if (private_mapping && (prot & PROT_EXEC)) {
-    /* We currently don't record the images that we
-     * exec(). Since we're being optimistic there (*cough*
-     * *cough*), we're doing no worse (in theory) by being
-     * optimistic about the shared libraries too, most of
-     * which are system libraries. */
+    /* Be optimistic about private executable mappings */
     LOG(debug) << "  (no copy for +x private mapping " << file_name << ")";
     return false;
   }
-  if (private_mapping && (0111 & stat->st_mode)) {
+  if (private_mapping && (0111 & stat.st_mode)) {
     /* A private mapping of an executable file usually
      * indicates mapping data sections of object files.
      * Since we're already assuming those change very
@@ -538,7 +511,7 @@ bool should_copy_mmap_region(const string& file_name, const struct stat* stat,
   // set*[gu]id(), the real answer may be different.
   bool can_write_file = (0 == access(file_name.c_str(), W_OK));
 
-  if (!can_write_file && 0 == stat->st_uid) {
+  if (!can_write_file && 0 == stat.st_uid) {
     // We would like to assert this, but on Ubuntu 13.10,
     // the file /lib/i386-linux-gnu/libdl-2.17.so is
     // writeable by root for unknown reasons.
@@ -569,7 +542,7 @@ bool should_copy_mmap_region(const string& file_name, const struct stat* stat,
     LOG(debug) << "  copying private mapping of non-system -x " << file_name;
     return true;
   }
-  if (!(0222 & stat->st_mode)) {
+  if (!(0222 & stat.st_mode)) {
     /* We couldn't write the file because it's read only.
      * But it's not a root-owned file (therefore not a
      * system file), so it's likely that it could be
@@ -582,29 +555,12 @@ bool should_copy_mmap_region(const string& file_name, const struct stat* stat,
      * irregular ... */
     FATAL() << "Unhandled mmap " << file_name << "(prot:" << HEX(prot)
             << ((flags & MAP_SHARED) ? ";SHARED" : "")
-            << "); uid:" << stat->st_uid << " mode:" << stat->st_mode;
+            << "); uid:" << stat.st_uid << " mode:" << stat.st_mode;
   }
   return true;
 }
 
-ScopedFd create_shmem_segment(const string& name, size_t num_bytes) {
-  char path[PATH_MAX];
-  snprintf(path, sizeof(path) - 1, "%s/%s", SHMEM_FS, name.c_str());
-
-  ScopedFd fd = open(path, O_CREAT | O_EXCL | O_RDWR | O_CLOEXEC, 0600);
-  if (0 > fd) {
-    FATAL() << "Failed to create shmem segment " << path;
-  }
-  /* Remove the fs name so that we don't have to worry about
-   * cleaning up this segment in error conditions. */
-  unlink(path);
-  resize_shmem_segment(fd, num_bytes);
-
-  LOG(debug) << "created shmem segment " << path;
-  return fd;
-}
-
-void resize_shmem_segment(ScopedFd& fd, size_t num_bytes) {
+void resize_shmem_segment(ScopedFd& fd, uint64_t num_bytes) {
   if (ftruncate(fd, num_bytes)) {
     FATAL() << "Failed to resize shmem to " << num_bytes;
   }
@@ -618,22 +574,6 @@ void cpuid(int code, int subrequest, unsigned int* a, unsigned int* c,
                : "ebx");
 }
 
-void set_cpu_affinity(int cpu) {
-  assert(cpu >= 0);
-
-  cpu_set_t mask;
-  CPU_ZERO(&mask);
-  CPU_SET(cpu, &mask);
-  if (0 > sched_setaffinity(0, sizeof(mask), &mask)) {
-    FATAL() << "Couldn't bind to CPU " << cpu;
-  }
-}
-
-int get_num_cpus() {
-  int cpus = (int)sysconf(_SC_NPROCESSORS_ONLN);
-  return cpus > 0 ? cpus : 1;
-}
-
 template <typename Arch>
 static void extract_clone_parameters_arch(const Registers& regs,
                                           remote_ptr<void>* stack,
@@ -721,69 +661,21 @@ void dump_task_map(const map<pid_t, Task*>& tasks) {
   printf("]\n");
 }
 
-static void set_syscall_result(Task* t, long ret) {
-  Registers r = t->regs();
-  r.set_syscall_result(ret);
-  t->set_regs(r);
+string real_path(const string& path) {
+  char buf[PATH_MAX];
+  if (realpath(path.c_str(), buf) == buf) {
+    return string(buf);
+  }
+  return path;
 }
 
-template <typename Arch>
-static void install_patched_seccomp_filter_arch(Task* t) {
-  // Take advantage of the fact that the filter program is arg3() in both
-  // prctl and seccomp syscalls.
-  bool ok = true;
-  auto prog =
-      t->read_mem(remote_ptr<typename Arch::sock_fprog>(t->regs().arg3()), &ok);
-  if (!ok) {
-    set_syscall_result(t, -EFAULT);
-    return;
+string exe_directory() {
+  string exe_path = real_path("/proc/self/exe");
+  int end = exe_path.length();
+  // Chop off the filename
+  while (end > 0 && exe_path[end - 1] != '/') {
+    --end;
   }
-  auto code = t->read_mem(prog.filter.rptr(), prog.len, &ok);
-  if (!ok) {
-    set_syscall_result(t, -EFAULT);
-    return;
-  }
-  for (auto& u : code) {
-    if (BPF_CLASS(u.code) == BPF_RET) {
-      // XXX If we need to support RET with A/X registers, we should
-      // extend the filter patch to dynamically cap the errno (if any) to
-      // max_errno and switch to SECCOMP_RET_TRACE.
-      ASSERT(t, BPF_RVAL(u.code) == BPF_K)
-          << "seccomp-bpf program uses BPF_RET with A/X register, not "
-             "supported";
-      if ((u.k & SECCOMP_RET_ACTION) == SECCOMP_RET_ERRNO) {
-        static int max_errno = 4095;
-        // The kernel caps the max errno to 4095, so we may as well do that
-        // here. This means filter data values > 4095 cannot be generated
-        // by this filter, which lets us disambiguate seccomp errno filter
-        // returns from our filter returns.
-        int filter_errno = min<int>(max_errno, u.k & SECCOMP_RET_DATA);
-        // Instead of forcing an errno return directly, trigger a ptrace
-        // trap so we can detect and handle it.
-        u.k = filter_errno | SECCOMP_RET_TRACE;
-      }
-    }
-  }
-
-  long ret;
-  {
-    AutoRemoteSyscalls remote(t);
-    AutoRestoreMem mem(remote, nullptr,
-                       sizeof(prog) +
-                           code.size() * sizeof(typename Arch::sock_filter));
-    auto code_ptr = mem.get().cast<typename Arch::sock_filter>();
-    t->write_mem(code_ptr, code.data(), code.size());
-    prog.filter = code_ptr;
-    auto prog_ptr = remote_ptr<void>(code_ptr + code.size())
-                        .cast<typename Arch::sock_fprog>();
-    t->write_mem(prog_ptr, prog);
-
-    ret = remote.syscall(t->regs().original_syscallno(), t->regs().arg1(),
-                         t->regs().arg2(), prog_ptr);
-  }
-  set_syscall_result(t, ret);
-}
-
-void install_patched_seccomp_filter(Task* t) {
-  RR_ARCH_FUNCTION(install_patched_seccomp_filter_arch, t->arch(), t);
+  exe_path.erase(end);
+  return exe_path;
 }
