@@ -13,10 +13,19 @@
 #include "ScopedFd.h"
 #include "TraceFrame.h"
 
+/*
+ * This file is a dumping ground for functionality that needs to be shared but
+ * has no other obvious place to go.
+ *
+ * We should minimize the amount of code here. Code that's only needed in one
+ * place can move out of this file.
+ */
+
+class KernelMapping;
 class Task;
 class TraceFrame;
 
-template <typename T, size_t N> constexpr size_t array_length(T (&array)[N]) {
+template <typename T, size_t N> constexpr size_t array_length(T(&array)[N]) {
   return N;
 }
 
@@ -48,25 +57,17 @@ template <typename T> bool type_has_no_holes() {
 }
 
 #define SHMEM_FS "/dev/shm"
-#define SHMEM_FS2 "/run/shm"
 
 /* The syscallbuf shared with tracees is created with this prefix
  * followed by the tracee tid, then immediately unlinked and shared
  * anonymously. */
-#define SYSCALLBUF_SHMEM_NAME_PREFIX "rr-tracee-shmem-"
-#define SYSCALLBUF_SHMEM_PATH_PREFIX SHMEM_FS "/" SYSCALLBUF_SHMEM_NAME_PREFIX
+#define SYSCALLBUF_SHMEM_PATH_PREFIX "/tmp/rr-tracee-shmem-"
 
 #define PREFIX_FOR_EMPTY_MMAPED_REGIONS "/tmp/rr-emptyfile-"
 
-enum Completion {
-  COMPLETE,
-  INCOMPLETE
-};
+enum Completion { COMPLETE, INCOMPLETE };
 
-enum Switchable {
-  PREVENT_SWITCH,
-  ALLOW_SWITCH
-};
+enum Switchable { PREVENT_SWITCH, ALLOW_SWITCH };
 
 /**
  * Create a file named |filename| and dump |buf_len| words in |buf| to
@@ -151,25 +152,9 @@ remote_ptr<void> floor_page_size(remote_ptr<void> addr);
 size_t page_size();
 
 /** Return the default action of |sig|. */
-enum signal_action {
-  DUMP_CORE,
-  TERMINATE,
-  CONTINUE,
-  STOP,
-  IGNORE
-};
+enum signal_action { DUMP_CORE, TERMINATE, CONTINUE, STOP, IGNORE };
 signal_action default_action(int sig);
 
-/**
- * Return true if |sig| may cause the status of other tasks to change
- * unpredictably beyond rr's observation.
- * 'deterministic' is true when the signal was delivered deterministically,
- * i.e. due to code execution as opposed to an asynchronous signal sent by some
- * process.
- */
-bool possibly_destabilizing_signal(Task* t, int sig,
-                                   SignalDeterministic deterministic);
-
 SignalDeterministic is_deterministic_signal(const siginfo_t& si);
 
 /**
@@ -180,20 +165,14 @@ SignalDeterministic is_deterministic_signal(const siginfo_t& si);
  * get away/ with not copying the region.  That doesn't mean it's
  * necessarily safe to skip copying!
  */
-bool should_copy_mmap_region(const std::string& filename,
-                             const struct stat* stat, int prot, int flags);
-
-/**
- * Return an fd referring to a new shmem segment with descriptive
- * |name| of size |num_bytes|.
- */
-ScopedFd create_shmem_segment(const std::string& name, size_t num_bytes);
+bool should_copy_mmap_region(const KernelMapping& mapping,
+                             const struct stat& stat);
 
 /**
  * Ensure that the shmem segment referred to by |fd| has exactly the
  * size |num_bytes|.
  */
-void resize_shmem_segment(ScopedFd& fd, size_t num_bytes);
+void resize_shmem_segment(ScopedFd& fd, uint64_t num_bytes);
 
 enum cpuid_requests {
   CPUID_GETVENDORSTRING,
@@ -217,17 +196,6 @@ enum cpuid_requests {
 void cpuid(int code, int subrequest, unsigned int* a, unsigned int* c,
            unsigned int* d);
 
-/**
- * Force this process (and its descendants) to only use the cpu with the given
- * index.
- */
-void set_cpu_affinity(int cpu);
-
-/**
- * Return the number of available CPUs in the system.
- */
-int get_num_cpus();
-
 /**
  * Extract various clone(2) parameters out of the given Task's registers.
  * Each remote_ptr parameter may be nullptr.
@@ -251,11 +219,8 @@ void dump_task_set(const std::set<Task*>& tasks);
 
 void dump_task_map(const std::map<pid_t, Task*>& tasks);
 
-/**
- * Assuming |t| is set up for a prctl or seccomp syscall that
- * installs a seccomp-bpf filter, patch the filter to signal the tracer instead
- * of silently delivering an errno, and install it.
- */
-void install_patched_seccomp_filter(Task* t);
+std::string real_path(const std::string& path);
+
+std::string exe_directory();
 
 #endif /* RR_UTIL_H_ */
